# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe._transfer import Transfer
    from stripe._transfer_reversal_service import TransferReversalService
    from stripe.params._transfer_create_params import TransferCreateParams
    from stripe.params._transfer_list_params import TransferListParams
    from stripe.params._transfer_retrieve_params import TransferRetrieveParams
    from stripe.params._transfer_update_params import TransferUpdateParams

_subservices = {
    "reversals": [
        "stripe._transfer_reversal_service",
        "TransferReversalService",
    ],
}


class TransferService(StripeService):
    reversals: "TransferReversalService"

    def __init__(self, requestor):
        super().__init__(requestor)

    def __getattr__(self, name):
        try:
            import_from, service = _subservices[name]
            service_class = getattr(
                import_module(import_from),
                service,
            )
            setattr(
                self,
                name,
                service_class(self._requestor),
            )
            return getattr(self, name)
        except KeyError:
            raise AttributeError()

    def list(
        self,
        params: Optional["TransferListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Transfer]":
        """
        Returns a list of existing transfers sent to connected accounts. The transfers are returned in sorted order, with the most recently created transfers appearing first.
        """
        return cast(
            "ListObject[Transfer]",
            self._request(
                "get",
                "/v1/transfers",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["TransferListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Transfer]":
        """
        Returns a list of existing transfers sent to connected accounts. The transfers are returned in sorted order, with the most recently created transfers appearing first.
        """
        return cast(
            "ListObject[Transfer]",
            await self._request_async(
                "get",
                "/v1/transfers",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "TransferCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Transfer":
        """
        To send funds from your Stripe account to a connected account, you create a new transfer object. Your [Stripe balance](https://docs.stripe.com/api#balance) must be able to cover the transfer amount, or you'll receive an “Insufficient Funds” error.
        """
        return cast(
            "Transfer",
            self._request(
                "post",
                "/v1/transfers",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "TransferCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Transfer":
        """
        To send funds from your Stripe account to a connected account, you create a new transfer object. Your [Stripe balance](https://docs.stripe.com/api#balance) must be able to cover the transfer amount, or you'll receive an “Insufficient Funds” error.
        """
        return cast(
            "Transfer",
            await self._request_async(
                "post",
                "/v1/transfers",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        transfer: str,
        params: Optional["TransferRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Transfer":
        """
        Retrieves the details of an existing transfer. Supply the unique transfer ID from either a transfer creation request or the transfer list, and Stripe will return the corresponding transfer information.
        """
        return cast(
            "Transfer",
            self._request(
                "get",
                "/v1/transfers/{transfer}".format(
                    transfer=sanitize_id(transfer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        transfer: str,
        params: Optional["TransferRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Transfer":
        """
        Retrieves the details of an existing transfer. Supply the unique transfer ID from either a transfer creation request or the transfer list, and Stripe will return the corresponding transfer information.
        """
        return cast(
            "Transfer",
            await self._request_async(
                "get",
                "/v1/transfers/{transfer}".format(
                    transfer=sanitize_id(transfer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        transfer: str,
        params: Optional["TransferUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Transfer":
        """
        Updates the specified transfer by setting the values of the parameters passed. Any parameters not provided will be left unchanged.

        This request accepts only metadata as an argument.
        """
        return cast(
            "Transfer",
            self._request(
                "post",
                "/v1/transfers/{transfer}".format(
                    transfer=sanitize_id(transfer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        transfer: str,
        params: Optional["TransferUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Transfer":
        """
        Updates the specified transfer by setting the values of the parameters passed. Any parameters not provided will be left unchanged.

        This request accepts only metadata as an argument.
        """
        return cast(
            "Transfer",
            await self._request_async(
                "post",
                "/v1/transfers/{transfer}".format(
                    transfer=sanitize_id(transfer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject, UntypedStripeObject
from stripe.v2._amount import Amount
from typing import ClassVar, List, Optional
from typing_extensions import Literal


class OutboundTransfer(StripeObject):
    """
    OutboundTransfer represents a single money movement from one FinancialAccount you own to a payout method you also own.
    """

    OBJECT_NAME: ClassVar[Literal["v2.money_management.outbound_transfer"]] = (
        "v2.money_management.outbound_transfer"
    )

    class DeliveryOptions(StripeObject):
        bank_account: Optional[Literal["automatic", "local", "wire"]]
        """
        Open Enum. Method for bank account.
        """

    class From(StripeObject):
        debited: Amount
        """
        The monetary amount debited from the sender, only set on responses.
        """
        financial_account: str
        """
        The FinancialAccount that funds were pulled from.
        """

    class StatusDetails(StripeObject):
        class Failed(StripeObject):
            reason: Literal[
                "fx_rate_drift_exceeded_after_review",
                "payout_method_amount_limit_exceeded",
                "payout_method_declined",
                "payout_method_does_not_exist",
                "payout_method_expired",
                "payout_method_unsupported",
                "payout_method_usage_frequency_limit_exceeded",
                "review_rejected",
                "unknown_failure",
            ]
            """
            Open Enum. The `failed` status reason.
            """

        class Processing(StripeObject):
            reason: Literal["under_review"]
            """
            Open Enum. The `processing` status reason.
            """

        class Returned(StripeObject):
            reason: Literal[
                "payout_method_canceled_by_customer",
                "payout_method_closed",
                "payout_method_currency_unsupported",
                "payout_method_does_not_exist",
                "payout_method_holder_address_incorrect",
                "payout_method_holder_details_incorrect",
                "payout_method_holder_name_incorrect",
                "payout_method_invalid_account_number",
                "payout_method_restricted",
                "recalled",
                "unknown_failure",
            ]
            """
            Open Enum. The `returned` status reason.
            """

        failed: Optional[Failed]
        """
        The `failed` status reason.
        """
        processing: Optional[Processing]
        """
        The `processing` status details.
        """
        returned: Optional[Returned]
        """
        The `returned` status reason.
        """
        _inner_class_types = {
            "failed": Failed,
            "processing": Processing,
            "returned": Returned,
        }

    class StatusTransitions(StripeObject):
        canceled_at: Optional[str]
        """
        Timestamp describing when an OutboundTransfer changed status to `canceled`.
        Represented as a RFC 3339 date & time UTC value in millisecond precision, for example: 2022-09-18T13:22:18.123Z.
        """
        failed_at: Optional[str]
        """
        Timestamp describing when an OutboundTransfer changed status to `failed`.
        Represented as a RFC 3339 date & time UTC value in millisecond precision, for example: 2022-09-18T13:22:18.123Z.
        """
        posted_at: Optional[str]
        """
        Timestamp describing when an OutboundTransfer changed status to `posted`.
        Represented as a RFC 3339 date & time UTC value in millisecond precision, for example: 2022-09-18T13:22:18.123Z.
        """
        returned_at: Optional[str]
        """
        Timestamp describing when an OutboundTransfer changed status to `returned`.
        Represented as a RFC 3339 date & time UTC value in millisecond precision, for example: 2022-09-18T13:22:18.123Z.
        """

    class To(StripeObject):
        class PayoutMethodOptions(StripeObject):
            class BankAccount(StripeObject):
                preferred_networks: List[
                    Literal[
                        "ach",
                        "becs",
                        "eft",
                        "fedwire",
                        "fps",
                        "npp",
                        "rtp",
                        "sepa_credit",
                        "sepa_instant",
                        "swift",
                    ]
                ]
                """
                The preferred networks to use for this OutboundTransfer.
                """

            bank_account: Optional[BankAccount]
            """
            Options for bank account payout methods.
            """
            _inner_class_types = {"bank_account": BankAccount}

        credited: Amount
        """
        The monetary amount being credited to the destination.
        """
        payout_method: str
        """
        The payout method which the OutboundTransfer uses to send payout.
        """
        payout_method_options: Optional[PayoutMethodOptions]
        """
        Payout method options for the OutboundTransfer.
        """
        _inner_class_types = {"payout_method_options": PayoutMethodOptions}

    class TraceId(StripeObject):
        status: Literal["pending", "supported", "unsupported"]
        """
        Possible values are `pending`, `supported`, and `unsupported`. Initially set to `pending`, it changes to
        `supported` when the recipient bank provides a trace ID, or `unsupported` if the recipient bank doesn't support it.
        Note that this status may not align with the OutboundPayment or OutboundTransfer status and can remain `pending`
        even after the payment or transfer is posted.
        """
        value: Optional[str]
        """
        The trace ID value if `trace_id.status` is `supported`, otherwise empty.
        """

    amount: Amount
    """
    The "presentment amount" for the OutboundTransfer.
    """
    cancelable: bool
    """
    Returns true if the OutboundTransfer can be canceled, and false otherwise.
    """
    created: str
    """
    Time at which the OutboundTransfer was created.
    Represented as a RFC 3339 date & time UTC value in millisecond precision, for example: 2022-09-18T13:22:18.123Z.
    """
    delivery_options: Optional[DeliveryOptions]
    """
    Delivery options to be used to send the OutboundTransfer.
    """
    description: Optional[str]
    """
    An arbitrary string attached to the OutboundTransfer. Often useful for displaying to users.
    """
    expected_arrival_date: Optional[str]
    """
    The date when funds are expected to arrive in the payout method. This field is not set if the payout method is in a `failed`, `canceled`, or `returned` state.
    Represented as a RFC 3339 date & time UTC value in millisecond precision, for example: 2022-09-18T13:22:18.123Z.
    """
    from_: From
    """
    The FinancialAccount that funds were pulled from.
    """
    id: str
    """
    Unique identifier for the OutboundTransfer.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    metadata: Optional[UntypedStripeObject[str]]
    """
    Set of key-value pairs that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    object: Literal["v2.money_management.outbound_transfer"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    receipt_url: Optional[str]
    """
    A link to the Stripe-hosted receipt for this OutboundTransfer. The receipt link remains active for 60 days from the OutboundTransfer creation date. After this period, the link will expire and the receipt url value will be null.
    """
    recipient_verification: Optional[str]
    """
    The recipient verification id for this OutboundTransfer. Only required for countries with regulatory mandates to verify recipient names before OutboundTransfer creation.
    """
    statement_descriptor: str
    """
    The description that appears on the receiving end for an OutboundTransfer (for example, bank statement for external bank transfer). It will default to `STRIPE` if not set on the account settings.
    """
    status: Literal["canceled", "failed", "posted", "processing", "returned"]
    """
    Closed Enum. Current status of the OutboundTransfer: `processing`, `failed`, `posted`, `returned`, `canceled`.
    An OutboundTransfer is `processing` if it has been created and is processing.
    The status changes to `posted` once the OutboundTransfer has been "confirmed" and funds have left the account, or to `failed` or `canceled`.
    If an OutboundTransfer fails to arrive at its payout method, its status will change to `returned`.
    """
    status_details: Optional[StatusDetails]
    """
    Status details for an OutboundTransfer in a `processing`, `failed`, or `returned` state.
    """
    status_transitions: Optional[StatusTransitions]
    """
    Hash containing timestamps of when the object transitioned to a particular status.
    """
    to: To
    """
    To which payout method the OutboundTransfer was sent.
    """
    trace_id: TraceId
    """
    A unique identifier that can be used to track this OutboundTransfer with recipient bank. Banks might call this a "reference number" or something similar.
    """
    _inner_class_types = {
        "delivery_options": DeliveryOptions,
        "from": From,
        "status_details": StatusDetails,
        "status_transitions": StatusTransitions,
        "to": To,
        "trace_id": TraceId,
    }
    _field_remappings = {"from_": "from"}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import UntypedStripeObject
from typing import Any, Dict
from typing_extensions import NotRequired, TypedDict


class ReportRunCreateParams(TypedDict):
    report: str
    """
    The unique identifier of the `Report` being requested.
    """
    report_parameters: "Dict[str, Any]|UntypedStripeObject[Any]"
    """
    A map of parameter names to values, specifying how the report should be customized.
    The accepted parameters depend on the specific `Report` being run.
    """
    result_options: NotRequired["ReportRunCreateParamsResultOptions"]
    """
    Optional settings to customize the results of the `ReportRun`.
    """


class ReportRunCreateParamsResultOptions(TypedDict):
    compress_file: NotRequired[bool]
    """
    If set, the generated results file will be compressed into a ZIP folder.
    This is useful for reducing file size and download time for large results.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing import List
from typing_extensions import Literal, NotRequired, TypedDict


class EventListParams(TypedDict):
    created: NotRequired["EventListParamsCreated"]
    """
    Set of filters to query events within a range of `created` timestamps.
    """
    include: NotRequired[List[Literal["reason.request.client"]]]
    """
    Additional fields to include in the response.
    """
    limit: NotRequired[int]
    """
    The page size.
    """
    object_id: NotRequired[str]
    """
    Primary object ID used to retrieve related events.
    """
    types: NotRequired[List[str]]
    """
    An array of up to 20 strings containing specific event names.
    """


class EventListParamsCreated(TypedDict):
    gt: NotRequired[str]
    """
    Filter for events created after the specified timestamp.
    """
    gte: NotRequired[str]
    """
    Filter for events created at or after the specified timestamp.
    """
    lt: NotRequired[str]
    """
    Filter for events created before the specified timestamp.
    """
    lte: NotRequired[str]
    """
    Filter for events created at or before the specified timestamp.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class PromotionCodeCreateParams(RequestOptions):
    active: NotRequired[bool]
    """
    Whether the promotion code is currently active.
    """
    code: NotRequired[str]
    """
    The customer-facing code. Regardless of case, this code must be unique across all active promotion codes for a specific customer. Valid characters are lower case letters (a-z), upper case letters (A-Z), digits (0-9), and dashes (-).

    If left blank, we will generate one automatically.
    """
    customer: NotRequired[str]
    """
    The customer who can use this promotion code. If not set, all customers can use the promotion code.
    """
    customer_account: NotRequired[str]
    """
    The account representing the customer who can use this promotion code. If not set, all customers can use the promotion code.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    expires_at: NotRequired[int]
    """
    The timestamp at which this promotion code will expire. If the coupon has specified a `redeems_by`, then this value cannot be after the coupon's `redeems_by`.
    """
    max_redemptions: NotRequired[int]
    """
    A positive integer specifying the number of times the promotion code can be redeemed. If the coupon has specified a `max_redemptions`, then this value cannot be greater than the coupon's `max_redemptions`.
    """
    metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    promotion: "PromotionCodeCreateParamsPromotion"
    """
    The promotion referenced by this promotion code.
    """
    restrictions: NotRequired["PromotionCodeCreateParamsRestrictions"]
    """
    Settings that restrict the redemption of the promotion code.
    """


class PromotionCodeCreateParamsPromotion(TypedDict):
    coupon: NotRequired[str]
    """
    If promotion `type` is `coupon`, the coupon for this promotion code.
    """
    type: Literal["coupon"]
    """
    Specifies the type of promotion.
    """


class PromotionCodeCreateParamsRestrictions(TypedDict):
    currency_options: NotRequired[
        "Dict[str, PromotionCodeCreateParamsRestrictionsCurrencyOptions]|UntypedStripeObject[PromotionCodeCreateParamsRestrictionsCurrencyOptions]"
    ]
    """
    Promotion codes defined in each available currency option. Each key must be a three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html) and a [supported currency](https://stripe.com/docs/currencies).
    """
    first_time_transaction: NotRequired[bool]
    """
    A Boolean indicating if the Promotion Code should only be redeemed for Customers without any successful payments or invoices
    """
    minimum_amount: NotRequired[int]
    """
    Minimum amount required to redeem this Promotion Code into a Coupon (e.g., a purchase must be $100 or more to work).
    """
    minimum_amount_currency: NotRequired[str]
    """
    Three-letter [ISO code](https://stripe.com/docs/currencies) for minimum_amount
    """


class PromotionCodeCreateParamsRestrictionsCurrencyOptions(TypedDict):
    minimum_amount: NotRequired[int]
    """
    Minimum amount required to redeem this Promotion Code into a Coupon (e.g., a purchase must be $100 or more to work).
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.crypto._customer import Customer
    from stripe.crypto._customer_consumer_wallet_service import (
        CustomerConsumerWalletService,
    )
    from stripe.crypto._customer_payment_token_service import (
        CustomerPaymentTokenService,
    )
    from stripe.params.crypto._customer_retrieve_params import (
        CustomerRetrieveParams,
    )

_subservices = {
    "consumer_wallets": [
        "stripe.crypto._customer_consumer_wallet_service",
        "CustomerConsumerWalletService",
    ],
    "payment_tokens": [
        "stripe.crypto._customer_payment_token_service",
        "CustomerPaymentTokenService",
    ],
}


class CustomerService(StripeService):
    consumer_wallets: "CustomerConsumerWalletService"
    payment_tokens: "CustomerPaymentTokenService"

    def __init__(self, requestor):
        super().__init__(requestor)

    def __getattr__(self, name):
        try:
            import_from, service = _subservices[name]
            service_class = getattr(
                import_module(import_from),
                service,
            )
            setattr(
                self,
                name,
                service_class(self._requestor),
            )
            return getattr(self, name)
        except KeyError:
            raise AttributeError()

    def retrieve(
        self,
        id: str,
        params: Optional["CustomerRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Customer":
        """
        Retrieves the details of a Crypto Customer.
        """
        return cast(
            "Customer",
            self._request(
                "get",
                "/v1/crypto/customers/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["CustomerRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Customer":
        """
        Retrieves the details of a Crypto Customer.
        """
        return cast(
            "Customer",
            await self._request_async(
                "get",
                "/v1/crypto/customers/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from decimal import Decimal
from stripe._stripe_object import StripeObject, UntypedStripeObject
from typing import ClassVar, List, Optional
from typing_extensions import Literal


class Cadence(StripeObject):
    """
    A Cadence represents a billing schedule applied to a customer, defining when and how often to generate invoices, such as on monthly or annual billing cycle.
    """

    OBJECT_NAME: ClassVar[Literal["v2.billing.cadence"]] = "v2.billing.cadence"

    class BillingCycle(StripeObject):
        class Day(StripeObject):
            class Time(StripeObject):
                hour: int
                """
                The hour at which the billing cycle ends.
                This must be an integer between 0 and 23, inclusive.
                0 represents midnight, and 23 represents 11 PM.
                """
                minute: int
                """
                The minute at which the billing cycle ends.
                Must be an integer between 0 and 59, inclusive.
                """
                second: Optional[int]
                """
                The second at which the billing cycle ends.
                Must be an integer between 0 and 59, inclusive.
                """

            time: Time
            """
            The time at which the billing cycle ends.
            """
            _inner_class_types = {"time": Time}

        class Month(StripeObject):
            class Time(StripeObject):
                hour: int
                """
                The hour at which the billing cycle ends.
                This must be an integer between 0 and 23, inclusive.
                0 represents midnight, and 23 represents 11 PM.
                """
                minute: int
                """
                The minute at which the billing cycle ends.
                Must be an integer between 0 and 59, inclusive.
                """
                second: Optional[int]
                """
                The second at which the billing cycle ends.
                Must be an integer between 0 and 59, inclusive.
                """

            day_of_month: int
            """
            The day to anchor the billing on for a type="month" billing cycle from 1-31.
            If this number is greater than the number of days in the month being billed,
            this anchors to the last day of the month.
            """
            month_of_year: Optional[int]
            """
            The month to anchor the billing on for a type="month" billing cycle from
            1-12. Occurrences are calculated from the month anchor.
            """
            time: Time
            """
            The time at which the billing cycle ends.
            """
            _inner_class_types = {"time": Time}

        class Week(StripeObject):
            class Time(StripeObject):
                hour: int
                """
                The hour at which the billing cycle ends.
                This must be an integer between 0 and 23, inclusive.
                0 represents midnight, and 23 represents 11 PM.
                """
                minute: int
                """
                The minute at which the billing cycle ends.
                Must be an integer between 0 and 59, inclusive.
                """
                second: Optional[int]
                """
                The second at which the billing cycle ends.
                Must be an integer between 0 and 59, inclusive.
                """

            day_of_week: int
            """
            The day of the week to bill the type=week billing cycle on.
            Numbered from 1-7 for Monday to Sunday respectively, based on the ISO-8601 week day numbering.
            """
            time: Time
            """
            The time at which the billing cycle ends.
            """
            _inner_class_types = {"time": Time}

        class Year(StripeObject):
            class Time(StripeObject):
                hour: int
                """
                The hour at which the billing cycle ends.
                This must be an integer between 0 and 23, inclusive.
                0 represents midnight, and 23 represents 11 PM.
                """
                minute: int
                """
                The minute at which the billing cycle ends.
                Must be an integer between 0 and 59, inclusive.
                """
                second: Optional[int]
                """
                The second at which the billing cycle ends.
                Must be an integer between 0 and 59, inclusive.
                """

            day_of_month: int
            """
            The day to anchor the billing on for a type="month" billing cycle from 1-31.
            If this number is greater than the number of days in the month being billed,
            this anchors to the last day of the month.
            """
            month_of_year: int
            """
            The month to bill on from 1-12. If not provided, this defaults to the month the cadence was created.
            """
            time: Time
            """
            The time at which the billing cycle ends.
            """
            _inner_class_types = {"time": Time}

        day: Optional[Day]
        """
        Specific configuration for determining billing dates when type=day.
        """
        interval_count: int
        """
        The number of intervals (specified in the interval attribute) between cadence billings. For example, type=month and interval_count=3 bills every 3 months.
        """
        month: Optional[Month]
        """
        Specific configuration for determining billing dates when type=month.
        """
        type: Literal["day", "month", "week", "year"]
        """
        The frequency at which a cadence bills.
        """
        week: Optional[Week]
        """
        Specific configuration for determining billing dates when type=week.
        """
        year: Optional[Year]
        """
        Specific configuration for determining billing dates when type=year.
        """
        _inner_class_types = {
            "day": Day,
            "month": Month,
            "week": Week,
            "year": Year,
        }

    class InvoiceDiscountRule(StripeObject):
        class PercentOff(StripeObject):
            class MaximumApplications(StripeObject):
                type: Literal["indefinite"]
                """
                Max applications type of this discount, ex: indefinite.
                """

            maximum_applications: MaximumApplications
            """
            The maximum applications configuration for this discount.
            """
            percent_off: Decimal
            """
            Percent that is taken off the amount. For example, a percent_off of 50.0 reduces a 100 USD amount to 50 USD.
            """
            _inner_class_types = {"maximum_applications": MaximumApplications}
            _field_encodings = {"percent_off": "decimal_string"}

        id: str
        """
        Unique identifier for the object.
        """
        percent_off: Optional[PercentOff]
        """
        Details if the discount is a percentage off.
        """
        type: Literal["percent_off"]
        """
        The type of the discount.
        """
        _inner_class_types = {"percent_off": PercentOff}

    class Payer(StripeObject):
        billing_profile: str
        """
        The ID of the Billing Profile object which determines how a bill is paid.
        """
        customer: Optional[str]
        """
        The ID of the Customer object.
        """
        type: Literal["customer"]
        """
        A string identifying the type of the payer. Currently the only supported value is `customer`.
        """

    class Settings(StripeObject):
        class Bill(StripeObject):
            id: str
            """
            The ID of the referenced settings object.
            """
            version: Optional[str]
            """
            Returns the Settings Version when the cadence is pinned to a specific version.
            """

        class Collection(StripeObject):
            id: str
            """
            The ID of the referenced settings object.
            """
            version: Optional[str]
            """
            Returns the Settings Version when the cadence is pinned to a specific version.
            """

        bill: Optional[Bill]
        """
        Settings that configure bills generation, which includes calculating totals, tax, and presenting invoices.
        """
        collection: Optional[Collection]
        """
        Settings that configure and manage the behavior of collecting payments.
        """
        _inner_class_types = {"bill": Bill, "collection": Collection}

    class SettingsData(StripeObject):
        class Bill(StripeObject):
            class Calculation(StripeObject):
                class Tax(StripeObject):
                    type: Literal["automatic", "manual"]
                    """
                    Determines if tax is calculated automatically based on a PTC or manually based on rules defined by the business. Defaults to "manual".
                    """

                tax: Optional[Tax]
                """
                Settings for calculating tax.
                """
                _inner_class_types = {"tax": Tax}

            class Invoice(StripeObject):
                class TimeUntilDue(StripeObject):
                    interval: Literal["day", "month", "week", "year"]
                    """
                    The interval unit for the time until due.
                    """
                    interval_count: int
                    """
                    The number of interval units. For example, if interval=day and interval_count=30,
                    the invoice is due in 30 days.
                    """

                time_until_due: Optional[TimeUntilDue]
                """
                The amount of time until the invoice is overdue for payment.
                """
                _inner_class_types = {"time_until_due": TimeUntilDue}

            calculation: Calculation
            """
            Settings related to calculating a bill.
            """
            invoice: Invoice
            """
            Settings related to invoice behavior.
            """
            invoice_rendering_template: Optional[str]
            """
            The ID of the invoice rendering template to be used when generating invoices.
            """
            _inner_class_types = {
                "calculation": Calculation,
                "invoice": Invoice,
            }

        class Collection(StripeObject):
            class EmailDelivery(StripeObject):
                class PaymentDue(StripeObject):
                    enabled: bool
                    """
                    If true an email for the invoice would be generated and sent out.
                    """
                    include_payment_link: bool
                    """
                    If true the payment link to hosted invoice page would be included in email and PDF of the invoice.
                    """

                payment_due: Optional[PaymentDue]
                """
                Controls emails for when the payment is due. For example after the invoice is finalized and transitions to Open state.
                """
                _inner_class_types = {"payment_due": PaymentDue}

            class PaymentMethodOptions(StripeObject):
                class AcssDebit(StripeObject):
                    class MandateOptions(StripeObject):
                        transaction_type: Optional[
                            Literal["business", "personal"]
                        ]
                        """
                        Transaction type of the mandate.
                        """

                    mandate_options: Optional[MandateOptions]
                    """
                    Additional fields for Mandate creation.
                    """
                    verification_method: Optional[
                        Literal["automatic", "instant", "microdeposits"]
                    ]
                    """
                    Verification method.
                    """
                    _inner_class_types = {"mandate_options": MandateOptions}

                class Bancontact(StripeObject):
                    preferred_language: Optional[
                        Literal["de", "en", "fr", "nl"]
                    ]
                    """
                    Preferred language of the Bancontact authorization page that the customer is redirected to.
                    """

                class Card(StripeObject):
                    class MandateOptions(StripeObject):
                        amount: Optional[int]
                        """
                        Amount to be charged for future payments.
                        """
                        amount_type: Optional[Literal["fixed", "maximum"]]
                        """
                        The AmountType for the mandate. One of `fixed` or `maximum`.
                        """
                        description: Optional[str]
                        """
                        A description of the mandate that is meant to be displayed to the customer.
                        """
                        _field_encodings = {"amount": "int64_string"}

                    mandate_options: Optional[MandateOptions]
                    """
                    Configuration options for setting up an eMandate for cards issued in India.
                    """
                    network: Optional[str]
                    """
                    Selected network to process the payment on. Depends on the available networks of the card.
                    """
                    request_three_d_secure: Optional[
                        Literal["any", "automatic", "challenge"]
                    ]
                    """
                    An advanced option 3D Secure. We strongly recommend that you rely on our SCA Engine to automatically prompt your customers
                    for authentication based on risk level and [other requirements](https://docs.stripe.com/strong-customer-authentication).
                    However, if you wish to request 3D Secure based on logic from your own fraud engine, provide this option.
                    Read our guide on [manually requesting 3D Secure](https://docs.stripe.com/payments/3d-secure/authentication-flow#manual-three-ds) for more information on how this configuration interacts with Radar and our SCA Engine.
                    """
                    _inner_class_types = {"mandate_options": MandateOptions}

                class CustomerBalance(StripeObject):
                    class BankTransfer(StripeObject):
                        class EuBankTransfer(StripeObject):
                            country: Literal[
                                "BE", "DE", "ES", "FR", "IE", "NL"
                            ]
                            """
                            The desired country code of the bank account information.
                            """

                        eu_bank_transfer: Optional[EuBankTransfer]
                        """
                        Configuration for `eu_bank_transfer` funding type. Required if `type` is `eu_bank_transfer`.
                        """
                        type: Optional[
                            Literal[
                                "eu_bank_transfer",
                                "gb_bank_transfer",
                                "jp_bank_transfer",
                                "mx_bank_transfer",
                                "us_bank_transfer",
                            ]
                        ]
                        """
                        The bank transfer type that can be used for funding.
                        """
                        _inner_class_types = {
                            "eu_bank_transfer": EuBankTransfer,
                        }

                    bank_transfer: Optional[BankTransfer]
                    """
                    Configuration for the bank transfer funding type, if the `funding_type` is set to `bank_transfer`.
                    """
                    funding_type: Optional[Literal["bank_transfer"]]
                    """
                    The funding method type to be used when there are not enough funds in the customer balance. Currently the only supported value is `bank_transfer`.
                    """
                    _inner_class_types = {"bank_transfer": BankTransfer}

                class Konbini(StripeObject):
                    pass

                class SepaDebit(StripeObject):
                    pass

                class UsBankAccount(StripeObject):
                    class FinancialConnections(StripeObject):
                        class Filters(StripeObject):
                            account_subcategories: List[
                                Literal["checking", "savings"]
                            ]
                            """
                            The account subcategories to use to filter for selectable accounts.
                            """

                        filters: Optional[Filters]
                        """
                        Provide filters for the linked accounts that the customer can select for the payment method.
                        """
                        permissions: List[
                            Literal[
                                "balances",
                                "ownership",
                                "payment_method",
                                "transactions",
                            ]
                        ]
                        """
                        The list of permissions to request. If this parameter is passed, the `payment_method` permission must be included.
                        """
                        prefetch: List[
                            Literal["balances", "ownership", "transactions"]
                        ]
                        """
                        List of data features that you would like to retrieve upon account creation.
                        """
                        _inner_class_types = {"filters": Filters}

                    financial_connections: FinancialConnections
                    """
                    Additional fields for Financial Connections Session creation.
                    """
                    verification_method: Literal[
                        "automatic", "instant", "microdeposits"
                    ]
                    """
                    Verification method.
                    """
                    _inner_class_types = {
                        "financial_connections": FinancialConnections,
                    }

                acss_debit: Optional[AcssDebit]
                """
                This sub-hash contains details about the Canadian pre-authorized debit payment method options.
                """
                bancontact: Optional[Bancontact]
                """
                This sub-hash contains details about the Bancontact payment method.
                """
                card: Optional[Card]
                """
                This sub-hash contains details about the Card payment method options.
                """
                customer_balance: Optional[CustomerBalance]
                """
                This sub-hash contains details about the Bank transfer payment method options.
                """
                konbini: Optional[Konbini]
                """
                This sub-hash contains details about the Konbini payment method options.
                """
                sepa_debit: Optional[SepaDebit]
                """
                This sub-hash contains details about the SEPA Direct Debit payment method options.
                """
                us_bank_account: Optional[UsBankAccount]
                """
                This sub-hash contains details about the ACH direct debit payment method options.
                """
                _inner_class_types = {
                    "acss_debit": AcssDebit,
                    "bancontact": Bancontact,
                    "card": Card,
                    "customer_balance": CustomerBalance,
                    "konbini": Konbini,
                    "sepa_debit": SepaDebit,
                    "us_bank_account": UsBankAccount,
                }

            collection_method: Literal["automatic", "send_invoice"]
            """
            Either automatic, or send_invoice. When charging automatically, Stripe attempts to pay this
            bill at the end of the period using the payment method attached to the billing profile. When sending an invoice,
            Stripe emails your billing profile an invoice with payment instructions.
            Defaults to automatic.
            """
            email_delivery: EmailDelivery
            """
            Email delivery settings.
            """
            payment_method_configuration: str
            """
            The ID of the PaymentMethodConfiguration object, which controls which payment methods are displayed to your customers.
            """
            payment_method_options: PaymentMethodOptions
            """
            Payment Method specific configuration stored on the object.
            """
            _inner_class_types = {
                "email_delivery": EmailDelivery,
                "payment_method_options": PaymentMethodOptions,
            }

        bill: Bill
        """
        Expanded bill settings data with actual configuration values.
        """
        collection: Collection
        """
        Expanded collection settings data with actual configuration values.
        """
        _inner_class_types = {"bill": Bill, "collection": Collection}

    billing_cycle: BillingCycle
    """
    The billing cycle is the object that defines future billing cycle dates.
    """
    created: str
    """
    Timestamp of when the object was created.
    """
    id: str
    """
    Unique identifier for the object.
    """
    invoice_discount_rules: Optional[List[InvoiceDiscountRule]]
    """
    The discount rules applied to all invoices for the cadence.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    lookup_key: Optional[str]
    """
    A lookup key used to retrieve cadences dynamically from a static string. Maximum length of 200 characters.
    """
    metadata: Optional[UntypedStripeObject[str]]
    """
    Set of [key-value pairs](https://docs.stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    next_billing_date: Optional[str]
    """
    The date that the billing cadence will next bill. Null if the cadence is not active.
    """
    object: Literal["v2.billing.cadence"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    payer: Payer
    """
    The payer determines the entity financially responsible for the bill.
    """
    settings: Optional[Settings]
    """
    The settings associated with the cadence.
    """
    settings_data: Optional[SettingsData]
    """
    Settings data that contains expanded billing settings configuration with actual values.
    """
    status: Literal["active", "canceled"]
    """
    The current status of the cadence.
    """
    test_clock: Optional[str]
    """
    The ID of the Test Clock.
    """
    _inner_class_types = {
        "billing_cycle": BillingCycle,
        "invoice_discount_rules": InvoiceDiscountRule,
        "payer": Payer,
        "settings": Settings,
        "settings_data": SettingsData,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import UntypedStripeObject
from stripe.v2._amount import AmountParam
from typing import Dict
from typing_extensions import NotRequired, TypedDict


class SettlementAllocationIntentCreateParams(TypedDict):
    amount: AmountParam
    """
    The amount and currency of the SettlementAllocationIntent. Allowed Currencies are `gbp` | `eur`.
    """
    expected_settlement_date: str
    """
    Date when we expect to receive the funds. Must be in future .
    """
    financial_account: str
    """
    Financial Account Id where the funds are expected for this SettlementAllocationIntent.
    """
    metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    Metadata associated with the SettlementAllocationIntent.
    """
    reference: str
    """
    Reference for the SettlementAllocationIntent. This should be same as the transaction reference used by payments processor to send funds to Stripe. Must have length between 5 and 255 characters and it must be unique among existing SettlementAllocationIntents that have a non-terminal status (`pending`, `submitted`, `matched`, `errored`).
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing_extensions import NotRequired, TypedDict


class ApprovalRequestListParams(TypedDict):
    limit: NotRequired[int]
    """
    Maximum number of results to return.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import List
from typing_extensions import Literal, NotRequired


class ReaderListParams(RequestOptions):
    device_type: NotRequired[
        Literal[
            "bbpos_chipper2x",
            "bbpos_wisepad3",
            "bbpos_wisepos_e",
            "mobile_phone_reader",
            "simulated_stripe_s700",
            "simulated_stripe_s710",
            "simulated_verifone_m425",
            "simulated_verifone_p630",
            "simulated_verifone_ux700",
            "simulated_verifone_v660p",
            "simulated_wisepos_e",
            "stripe_m2",
            "stripe_s700",
            "stripe_s710",
            "verifone_P400",
            "verifone_m425",
            "verifone_p630",
            "verifone_ux700",
            "verifone_v660p",
        ]
    ]
    """
    Filters readers by device type
    """
    ending_before: NotRequired[str]
    """
    A cursor for use in pagination. `ending_before` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, starting with `obj_bar`, your subsequent call can include `ending_before=obj_bar` in order to fetch the previous page of the list.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    limit: NotRequired[int]
    """
    A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 10.
    """
    location: NotRequired[str]
    """
    A location ID to filter the response list to only readers at the specific location
    """
    serial_number: NotRequired[str]
    """
    Filters readers by serial number
    """
    starting_after: NotRequired[str]
    """
    A cursor for use in pagination. `starting_after` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with `obj_foo`, your subsequent call can include `starting_after=obj_foo` in order to fetch the next page of the list.
    """
    status: NotRequired[Literal["offline", "online"]]
    """
    A status filter to filter readers to only offline or online readers
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject, UntypedStripeObject
from stripe.v2._amount import Amount
from typing import ClassVar, List, Optional
from typing_extensions import Literal


class FinancialAccount(StripeObject):
    """
    A FinancialAccount represents a balance and can be used as the source or destination for the money management (`/v2/money_management`) APIs.
    """

    OBJECT_NAME: ClassVar[Literal["v2.money_management.financial_account"]] = (
        "v2.money_management.financial_account"
    )

    class AccruedFees(StripeObject):
        currencies: List[str]
        """
        The currencies enabled for fee accrual on this FinancialAccount.
        """
        direction: Literal["payable", "receivable"]
        """
        Direction of fee accrual for this FinancialAccount.
        """

    class Balance(StripeObject):
        available: UntypedStripeObject[Amount]
        """
        Balance that can be used for money movement.
        """
        inbound_pending: UntypedStripeObject[Amount]
        """
        Balance of inbound funds that will later transition to the `available` balance.
        """
        outbound_pending: UntypedStripeObject[Amount]
        """
        Balance of funds that are being used for a pending outbound money movement.
        """

    class Credit(StripeObject):
        class FundedBy(StripeObject):
            class Platform(StripeObject):
                financial_account: str
                """
                The platform FinancialAccount used to fund this credit FinancialAccount.
                """

            platform: Platform
            """
            Details for platform-funded credit FinancialAccounts.
            """
            type: Literal["platform", "stripe"]
            """
            The type of funding source for this credit FinancialAccount.
            """
            _inner_class_types = {"platform": Platform}

        funded_by: Optional[FundedBy]
        """
        Details about how this credit FinancialAccount is funded.
        """
        supported_currencies: List[str]
        """
        The currencies supported by this credit FinancialAccount.
        """
        _inner_class_types = {"funded_by": FundedBy}

    class ManagedBy(StripeObject):
        type: Literal["multiprocessor_settlement"]
        """
        Enum describing the Stripe product that is managing this FinancialAccount.
        """

    class MultiprocessorSettlement(StripeObject):
        settlement_currencies: List[str]
        """
        Settlement currencies enabled for this FinancialAccount.
        """

    class Other(StripeObject):
        type: str
        """
        The type of the FinancialAccount, represented as a string. Upgrade your API version to see the type reflected in `financial_account.type`.
        """

    class Payments(StripeObject):
        class BalanceByFundsType(StripeObject):
            class PaymentProcessing(StripeObject):
                available: UntypedStripeObject[Amount]
                """
                Balance that can be used for money movement.
                """
                inbound_pending: UntypedStripeObject[Amount]
                """
                Balance of inbound funds that will later transition to the `available` balance.
                """
                outbound_pending: UntypedStripeObject[Amount]
                """
                Balance of funds that are being used for a pending outbound money movement.
                """

            class StoredValue(StripeObject):
                available: UntypedStripeObject[Amount]
                """
                Balance that can be used for money movement.
                """
                inbound_pending: UntypedStripeObject[Amount]
                """
                Balance of inbound funds that will later transition to the `available` balance.
                """
                outbound_pending: UntypedStripeObject[Amount]
                """
                Balance of funds that are being used for a pending outbound money movement.
                """

            payment_processing: PaymentProcessing
            """
            Payment processing funds are those that are received for goods or services and may only be used for payouts to self. These funds may be converted to stored value funds.
            """
            stored_value: StoredValue
            """
            Stored value funds may be used for either payouts to self or payments to others.
            """
            _inner_class_types = {
                "payment_processing": PaymentProcessing,
                "stored_value": StoredValue,
            }

        class StartingBalance(StripeObject):
            at: str
            """
            When the balance was projected.
            """
            available: UntypedStripeObject[Amount]
            """
            The available balance at the time when the balance was projected.
            """

        balance_by_funds_type: Optional[BalanceByFundsType]
        """
        The balance of the `payments` FinancialAccount is a mix of payment processing and stored value funds, and this field
        describes the breakdown between the two. The sum will match the balance of the FinancialAccount.
        """
        default_currency: str
        """
        The currency that non-settlement currency payments will be converted to.
        """
        settlement_currencies: List[str]
        """
        Settlement currencies enabled for this FinancialAccount. Payments in other currencies will be automatically converted to `default_currency`.
        """
        starting_balance: Optional[StartingBalance]
        """
        Describes the available balance when it was projected.
        """
        _inner_class_types = {
            "balance_by_funds_type": BalanceByFundsType,
            "starting_balance": StartingBalance,
        }

    class StatusDetails(StripeObject):
        class Closed(StripeObject):
            class ForwardingSettings(StripeObject):
                payment_method: Optional[str]
                """
                The address to send forwarded payments to.
                """
                payout_method: Optional[str]
                """
                The address to send forwarded payouts to.
                """

            forwarding_settings: Optional[ForwardingSettings]
            """
            The forwarding settings for the closed FinancialAccount.
            """
            reason: Literal["account_closed", "closed_by_platform", "other"]
            """
            The reason the FinancialAccount was closed.
            """
            _inner_class_types = {"forwarding_settings": ForwardingSettings}

        closed: Optional[Closed]
        """
        Details related to the closed state of the FinancialAccount.
        """
        _inner_class_types = {"closed": Closed}

    class Storage(StripeObject):
        funds_usage_type: Optional[Literal["business", "consumer"]]
        """
        The usage type for funds in this FinancialAccount. Can be used to specify that the funds are for Consumer activity.
        """
        holds_currencies: List[str]
        """
        The currencies that this FinancialAccount can hold.
        """

    accrued_fees: Optional[AccruedFees]
    """
    If this is a `accrued_fees` FinancialAccount, this hash include details specific to `accrued_fees` FinancialAccount.
    """
    balance: Balance
    """
    Multi-currency balance of this FinancialAccount, split by availability state. Each balance is represented as a hash where the key is the three-letter ISO currency code, in lowercase, and the value is the amount for that currency.
    """
    country: str
    """
    Open Enum. Two-letter country code that represents the country where the LegalEntity associated with the FinancialAccount is based in.
    """
    created: str
    """
    Time at which the object was created.
    """
    credit: Optional[Credit]
    """
    If this is a `credit` FinancialAccount, this hash includes details specific to `credit` FinancialAccounts.
    """
    display_name: Optional[str]
    """
    A descriptive name for the FinancialAccount, up to 50 characters long. This name will be used in the Stripe Dashboard and embedded components.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    managed_by: Optional[ManagedBy]
    """
    If this is a managed FinancialAccount, `managed_by` indicates the product that created and manages this FinancialAccount. For managed FinancialAccounts,
    creation of money management resources can only be orchestrated by the managing product.
    """
    metadata: Optional[UntypedStripeObject[str]]
    """
    Metadata associated with the FinancialAccount.
    """
    multiprocessor_settlement: Optional[MultiprocessorSettlement]
    """
    If this is a `multiprocessor_settlement` FinancialAccount, this hash includes details specific to `multiprocessor_settlement` FinancialAccounts.
    """
    object: Literal["v2.money_management.financial_account"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    other: Optional[Other]
    """
    If this is a `other` FinancialAccount, this hash indicates what the actual type is. Upgrade your API version to see it reflected in `type`.
    """
    payments: Optional[Payments]
    """
    If this is a `payments` FinancialAccount, this hash include details specific to `payments` FinancialAccount.
    """
    status: Literal["closed", "open", "pending"]
    """
    Closed Enum. An enum representing the status of the FinancialAccount. This indicates whether or not the FinancialAccount can be used for any money movement flows.
    """
    status_details: Optional[StatusDetails]
    """
    Additional details related to the status of the FinancialAccount.
    """
    storage: Optional[Storage]
    """
    If this is a `storage` FinancialAccount, this hash includes details specific to `storage` FinancialAccounts.
    """
    type: Literal[
        "accrued_fees",
        "credit",
        "multiprocessor_settlement",
        "other",
        "payments",
        "storage",
    ]
    """
    Type of the FinancialAccount. An additional hash is included on the FinancialAccount with a name matching this value.
    It contains additional information specific to the FinancialAccount type.
    """
    _inner_class_types = {
        "accrued_fees": AccruedFees,
        "balance": Balance,
        "credit": Credit,
        "managed_by": ManagedBy,
        "multiprocessor_settlement": MultiprocessorSettlement,
        "other": Other,
        "payments": Payments,
        "status_details": StatusDetails,
        "storage": Storage,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._createable_api_resource import CreateableAPIResource
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._stripe_object import UntypedStripeObject
from stripe._updateable_api_resource import UpdateableAPIResource
from stripe._util import sanitize_id
from typing import ClassVar, Optional, cast
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params._margin_create_params import MarginCreateParams
    from stripe.params._margin_list_params import MarginListParams
    from stripe.params._margin_modify_params import MarginModifyParams
    from stripe.params._margin_retrieve_params import MarginRetrieveParams


class Margin(
    CreateableAPIResource["Margin"],
    ListableAPIResource["Margin"],
    UpdateableAPIResource["Margin"],
):
    """
    A (partner) margin represents a specific discount distributed in partner reseller programs to business partners who
    resell products and services and earn a discount (margin) for doing so.
    """

    OBJECT_NAME: ClassVar[Literal["margin"]] = "margin"
    active: bool
    """
    Whether the margin can be applied to invoices, invoice items, or invoice line items. Defaults to `true`.
    """
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    metadata: Optional[UntypedStripeObject[str]]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    name: Optional[str]
    """
    Name of the margin that's displayed on, for example, invoices.
    """
    object: Literal["margin"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    percent_off: float
    """
    Percent that will be taken off the subtotal before tax (after all other discounts and promotions) of any invoice to which the margin is applied.
    """
    updated: int
    """
    Time at which the object was last updated. Measured in seconds since the Unix epoch.
    """

    @classmethod
    def create(cls, **params: Unpack["MarginCreateParams"]) -> "Margin":
        """
        Create a margin object to be used with invoices, invoice items, and invoice line items for a customer to represent a partner discount. A margin has a percent_off which is the percent that will be taken off the subtotal after all items and other discounts and promotions) of any invoices for a customer. Calculation of prorations do not include any partner margins applied on the original invoice item.
        """
        return cast(
            "Margin",
            cls._static_request(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    async def create_async(
        cls, **params: Unpack["MarginCreateParams"]
    ) -> "Margin":
        """
        Create a margin object to be used with invoices, invoice items, and invoice line items for a customer to represent a partner discount. A margin has a percent_off which is the percent that will be taken off the subtotal after all items and other discounts and promotions) of any invoices for a customer. Calculation of prorations do not include any partner margins applied on the original invoice item.
        """
        return cast(
            "Margin",
            await cls._static_request_async(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    def list(
        cls, **params: Unpack["MarginListParams"]
    ) -> ListObject["Margin"]:
        """
        Retrieve a list of your margins.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["MarginListParams"]
    ) -> ListObject["Margin"]:
        """
        Retrieve a list of your margins.
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def modify(
        cls, id: str, **params: Unpack["MarginModifyParams"]
    ) -> "Margin":
        """
        Update the specified margin object. Certain fields of the margin object are not editable.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "Margin",
            cls._static_request(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    async def modify_async(
        cls, id: str, **params: Unpack["MarginModifyParams"]
    ) -> "Margin":
        """
        Update the specified margin object. Certain fields of the margin object are not editable.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "Margin",
            await cls._static_request_async(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["MarginRetrieveParams"]
    ) -> "Margin":
        """
        Retrieve a margin object with the given ID.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["MarginRetrieveParams"]
    ) -> "Margin":
        """
        Retrieve a margin object with the given ID.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    @classmethod
    def class_url(cls):
        return "/v1/billing/margins"



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.v2.money_management._payout_intent_cancel_params import (
        PayoutIntentCancelParams,
    )
    from stripe.params.v2.money_management._payout_intent_create_params import (
        PayoutIntentCreateParams,
    )
    from stripe.params.v2.money_management._payout_intent_list_params import (
        PayoutIntentListParams,
    )
    from stripe.params.v2.money_management._payout_intent_retrieve_params import (
        PayoutIntentRetrieveParams,
    )
    from stripe.params.v2.money_management._payout_intent_update_params import (
        PayoutIntentUpdateParams,
    )
    from stripe.v2._list_object import ListObject
    from stripe.v2.money_management._payout_intent import PayoutIntent


class PayoutIntentService(StripeService):
    def list(
        self,
        params: Optional["PayoutIntentListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[PayoutIntent]":
        """
        Returns a list of PayoutIntents.
        """
        return cast(
            "ListObject[PayoutIntent]",
            self._request(
                "get",
                "/v2/money_management/payout_intents",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["PayoutIntentListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[PayoutIntent]":
        """
        Returns a list of PayoutIntents.
        """
        return cast(
            "ListObject[PayoutIntent]",
            await self._request_async(
                "get",
                "/v2/money_management/payout_intents",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "PayoutIntentCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "PayoutIntent":
        """
        Creates a PayoutIntent.
        """
        return cast(
            "PayoutIntent",
            self._request(
                "post",
                "/v2/money_management/payout_intents",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "PayoutIntentCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "PayoutIntent":
        """
        Creates a PayoutIntent.
        """
        return cast(
            "PayoutIntent",
            await self._request_async(
                "post",
                "/v2/money_management/payout_intents",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["PayoutIntentRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PayoutIntent":
        """
        Retrieves the details of an existing PayoutIntent by passing the unique PayoutIntent ID.
        """
        return cast(
            "PayoutIntent",
            self._request(
                "get",
                "/v2/money_management/payout_intents/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["PayoutIntentRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PayoutIntent":
        """
        Retrieves the details of an existing PayoutIntent by passing the unique PayoutIntent ID.
        """
        return cast(
            "PayoutIntent",
            await self._request_async(
                "get",
                "/v2/money_management/payout_intents/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        id: str,
        params: Optional["PayoutIntentUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PayoutIntent":
        """
        Updates a PayoutIntent. Only pending or requires_action PayoutIntents that are editable can be updated.
        """
        return cast(
            "PayoutIntent",
            self._request(
                "post",
                "/v2/money_management/payout_intents/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        id: str,
        params: Optional["PayoutIntentUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PayoutIntent":
        """
        Updates a PayoutIntent. Only pending or requires_action PayoutIntents that are editable can be updated.
        """
        return cast(
            "PayoutIntent",
            await self._request_async(
                "post",
                "/v2/money_management/payout_intents/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def cancel(
        self,
        id: str,
        params: Optional["PayoutIntentCancelParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PayoutIntent":
        """
        Cancels a PayoutIntent. Only pending PayoutIntents or processing PayoutIntents with cancelable OutboundPayment/Transfer can be canceled.
        """
        return cast(
            "PayoutIntent",
            self._request(
                "post",
                "/v2/money_management/payout_intents/{id}/cancel".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def cancel_async(
        self,
        id: str,
        params: Optional["PayoutIntentCancelParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PayoutIntent":
        """
        Cancels a PayoutIntent. Only pending PayoutIntents or processing PayoutIntents with cancelable OutboundPayment/Transfer can be canceled.
        """
        return cast(
            "PayoutIntent",
            await self._request_async(
                "post",
                "/v2/money_management/payout_intents/{id}/cancel".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing_extensions import Literal, NotRequired, TypedDict


class UsBankAccountCreateParams(TypedDict):
    account_number: str
    """
    The account number of the bank account.
    """
    bank_account_type: NotRequired[Literal["checking", "savings"]]
    """
    Closed Enum. The type of the bank account (checking or savings).
    """
    currency: str
    """
    The currency of the bank account.
    """
    fedwire_routing_number: NotRequired[str]
    """
    The fedwire routing number of the bank account. Note that certain banks have the same ACH and wire routing number.
    """
    routing_number: NotRequired[str]
    """
    The ACH routing number of the bank account. Note that certain banks have the same ACH and wire routing number.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.shared_payment._granted_token_retrieve_params import (
        GrantedTokenRetrieveParams,
    )
    from stripe.shared_payment._granted_token import GrantedToken


class GrantedTokenService(StripeService):
    def retrieve(
        self,
        shared_payment_granted_token: str,
        params: Optional["GrantedTokenRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "GrantedToken":
        """
        Retrieves an existing SharedPaymentGrantedToken object
        """
        return cast(
            "GrantedToken",
            self._request(
                "get",
                "/v1/shared_payment/granted_tokens/{shared_payment_granted_token}".format(
                    shared_payment_granted_token=sanitize_id(
                        shared_payment_granted_token
                    ),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        shared_payment_granted_token: str,
        params: Optional["GrantedTokenRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "GrantedToken":
        """
        Retrieves an existing SharedPaymentGrantedToken object
        """
        return cast(
            "GrantedToken",
            await self._request_async(
                "get",
                "/v1/shared_payment/granted_tokens/{shared_payment_granted_token}".format(
                    shared_payment_granted_token=sanitize_id(
                        shared_payment_granted_token
                    ),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import UntypedStripeObject
from typing import Dict
from typing_extensions import NotRequired, TypedDict


class ProfileCreateParams(TypedDict):
    customer: str
    """
    The ID of the customer object.
    """
    default_payment_method: NotRequired[str]
    """
    The ID of the payment method object.
    """
    display_name: NotRequired[str]
    """
    A customer-facing name for the billing profile.
    Maximum length of 250 characters.
    """
    lookup_key: NotRequired[str]
    """
    An internal key you can use to search for a particular billing profile. It must be unique among billing profiles for a given customer.
    Maximum length of 200 characters.
    """
    metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    Set of key-value pairs that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.capital._financing_offer import (
        FinancingOffer as FinancingOffer,
    )
    from stripe.capital._financing_offer_service import (
        FinancingOfferService as FinancingOfferService,
    )
    from stripe.capital._financing_summary import (
        FinancingSummary as FinancingSummary,
    )
    from stripe.capital._financing_summary_service import (
        FinancingSummaryService as FinancingSummaryService,
    )
    from stripe.capital._financing_transaction import (
        FinancingTransaction as FinancingTransaction,
    )
    from stripe.capital._financing_transaction_service import (
        FinancingTransactionService as FinancingTransactionService,
    )

# name -> (import_target, is_submodule)
_import_map = {
    "FinancingOffer": ("stripe.capital._financing_offer", False),
    "FinancingOfferService": (
        "stripe.capital._financing_offer_service",
        False,
    ),
    "FinancingSummary": ("stripe.capital._financing_summary", False),
    "FinancingSummaryService": (
        "stripe.capital._financing_summary_service",
        False,
    ),
    "FinancingTransaction": ("stripe.capital._financing_transaction", False),
    "FinancingTransactionService": (
        "stripe.capital._financing_transaction_service",
        False,
    ),
}
if not TYPE_CHECKING:

    def __getattr__(name):
        try:
            target, is_submodule = _import_map[name]
            module = import_module(target)
            if is_submodule:
                return module

            return getattr(
                module,
                name,
            )
        except KeyError:
            raise AttributeError()



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._gift_card_operation import GiftCardOperation
    from stripe._request_options import RequestOptions
    from stripe.params._gift_card_operation_retrieve_params import (
        GiftCardOperationRetrieveParams,
    )


class GiftCardOperationService(StripeService):
    def retrieve(
        self,
        id: str,
        params: Optional["GiftCardOperationRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "GiftCardOperation":
        """
        Retrieves a third-party gift card operation object.
        """
        return cast(
            "GiftCardOperation",
            self._request(
                "get",
                "/v1/gift_card_operations/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["GiftCardOperationRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "GiftCardOperation":
        """
        Retrieves a third-party gift card operation object.
        """
        return cast(
            "GiftCardOperation",
            await self._request_async(
                "get",
                "/v1/gift_card_operations/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._stripe_object import StripeObject, UntypedStripeObject
from stripe._test_helpers import APIResourceTestHelpers
from stripe._util import class_method_variant, sanitize_id
from typing import ClassVar, Optional, cast, overload
from typing_extensions import Literal, Type, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params.capital._financing_offer_create_params import (
        FinancingOfferCreateParams,
    )
    from stripe.params.capital._financing_offer_list_params import (
        FinancingOfferListParams,
    )
    from stripe.params.capital._financing_offer_mark_delivered_params import (
        FinancingOfferMarkDeliveredParams,
    )
    from stripe.params.capital._financing_offer_refill_params import (
        FinancingOfferRefillParams,
    )
    from stripe.params.capital._financing_offer_retrieve_params import (
        FinancingOfferRetrieveParams,
    )


class FinancingOffer(ListableAPIResource["FinancingOffer"]):
    """
    This is an object representing an offer of financing from
    Stripe Capital to a Connect subaccount.
    """

    OBJECT_NAME: ClassVar[Literal["capital.financing_offer"]] = (
        "capital.financing_offer"
    )

    class AcceptedTerms(StripeObject):
        advance_amount: int
        """
        Amount of financing offered, in minor units. For example, 1,000 USD is represented as 100000.
        """
        currency: str
        """
        Currency that the financing offer is transacted in. For example, `usd`.
        """
        fee_amount: int
        """
        Fixed fee amount, in minor units. For example, 100 USD is represented as 10000.
        """
        previous_financing_fee_discount_amount: Optional[int]
        """
        Populated when the `product_type` of the `financingoffer` is `refill`.
        Represents the discount amount on remaining premium for the existing loan at payout time.
        """
        withhold_rate: float
        """
        Per-transaction rate at which Stripe withholds funds to repay the financing.
        """

    class OfferedTerms(StripeObject):
        advance_amount: int
        """
        Amount of financing offered, in minor units. For example, 1,000 USD is represented as 100000.
        """
        campaign_type: Literal[
            "newly_eligible_user", "previously_eligible_user", "repeat_user"
        ]
        """
        Describes the type of user the offer is being extended to.
        """
        currency: str
        """
        Currency that the financing offer is transacted in. For example, `usd`.
        """
        fee_amount: int
        """
        Fixed fee amount, in minor units. For example, 100 USD is represented as 10000.
        """
        previous_financing_fee_discount_rate: Optional[float]
        """
        Populated when the `product_type` of the `financingoffer` is `refill`.
        Represents the discount rate percentage on remaining fee on the existing loan. When the `financing_offer`
        is paid out, the `previous_financing_fee_discount_amount` will be computed as the multiple of this rate
        and the remaining fee.
        """
        withhold_rate: float
        """
        Per-transaction rate at which Stripe withholds funds to repay the financing.
        """

    accepted_terms: Optional[AcceptedTerms]
    """
    This is an object representing the terms of an offer of financing from
    Stripe Capital to a Connected account. This resource represents
    the terms accepted by the Connected account, which may differ from those
    offered.
    """
    account: str
    """
    The ID of the merchant associated with this financing object.
    """
    charged_off_at: Optional[int]
    """
    The time at which this financing offer was charged off, if applicable. Given in seconds since unix epoch.
    """
    created: int
    """
    Time at which the offer was created. Given in seconds since unix epoch.
    """
    disclaimer_variant: Optional[
        Literal[
            "celtic_us_loan",
            "fundbox_au_financing",
            "youlend_de_financing",
            "youlend_fr_financing",
            "youlend_uk_mca",
            "youlend_us_mca",
        ]
    ]
    """
    The type of disclaimer to use for a financing offer in user-facing surfaces. The corresponding disclaimer text to use for each disclaimer_variant value can be found in the [marketing docs](https://docs.stripe.com/capital/marketing).
    """
    expires_after: float
    """
    Time at which the offer expires. Given in seconds since unix epoch.
    """
    financing_type: Optional[Literal["cash_advance", "flex_loan"]]
    """
    The type of financing being offered.
    """
    id: str
    """
    A unique identifier for the financing object.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    metadata: Optional[UntypedStripeObject[str]]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    object: Literal["capital.financing_offer"]
    """
    The object type: financing_offer.
    """
    offered_terms: Optional[OfferedTerms]
    """
    This is an object representing the terms of an offer of financing from
    Stripe Capital to a Connected account. This resource represents
    both the terms offered to the Connected account.
    """
    product_type: Optional[Literal["refill", "standard"]]
    """
    Financing product identifier.
    """
    replacement: Optional[str]
    """
    The ID of the financing offer that replaced this offer.
    """
    replacement_for: Optional[str]
    """
    The ID of the financing offer that this offer is a replacement for.
    """
    status: Literal[
        "accepted",
        "canceled",
        "completed",
        "delivered",
        "expired",
        "fully_repaid",
        "paid_out",
        "rejected",
        "replaced",
        "undelivered",
    ]
    """
    The current status of the offer.
    """
    type: Optional[Literal["cash_advance", "fixed_term_loan", "flex_loan"]]
    """
    See [financing_type](https://docs.stripe.com/api/capital/connect_financing_object#financing_offer_object-financing_type).
    """

    @classmethod
    def list(
        cls, **params: Unpack["FinancingOfferListParams"]
    ) -> ListObject["FinancingOffer"]:
        """
        Retrieves the financing offers available for Connected accounts that belong to your platform.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["FinancingOfferListParams"]
    ) -> ListObject["FinancingOffer"]:
        """
        Retrieves the financing offers available for Connected accounts that belong to your platform.
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def _cls_mark_delivered(
        cls,
        financing_offer: str,
        **params: Unpack["FinancingOfferMarkDeliveredParams"],
    ) -> "FinancingOffer":
        """
        Acknowledges that platform has received and delivered the financing_offer to
        the intended merchant recipient.
        """
        return cast(
            "FinancingOffer",
            cls._static_request(
                "post",
                "/v1/capital/financing_offers/{financing_offer}/mark_delivered".format(
                    financing_offer=sanitize_id(financing_offer)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    def mark_delivered(
        financing_offer: str,
        **params: Unpack["FinancingOfferMarkDeliveredParams"],
    ) -> "FinancingOffer":
        """
        Acknowledges that platform has received and delivered the financing_offer to
        the intended merchant recipient.
        """
        ...

    @overload
    def mark_delivered(
        self, **params: Unpack["FinancingOfferMarkDeliveredParams"]
    ) -> "FinancingOffer":
        """
        Acknowledges that platform has received and delivered the financing_offer to
        the intended merchant recipient.
        """
        ...

    @class_method_variant("_cls_mark_delivered")
    def mark_delivered(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["FinancingOfferMarkDeliveredParams"]
    ) -> "FinancingOffer":
        """
        Acknowledges that platform has received and delivered the financing_offer to
        the intended merchant recipient.
        """
        return cast(
            "FinancingOffer",
            self._request(
                "post",
                "/v1/capital/financing_offers/{financing_offer}/mark_delivered".format(
                    financing_offer=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    async def _cls_mark_delivered_async(
        cls,
        financing_offer: str,
        **params: Unpack["FinancingOfferMarkDeliveredParams"],
    ) -> "FinancingOffer":
        """
        Acknowledges that platform has received and delivered the financing_offer to
        the intended merchant recipient.
        """
        return cast(
            "FinancingOffer",
            await cls._static_request_async(
                "post",
                "/v1/capital/financing_offers/{financing_offer}/mark_delivered".format(
                    financing_offer=sanitize_id(financing_offer)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def mark_delivered_async(
        financing_offer: str,
        **params: Unpack["FinancingOfferMarkDeliveredParams"],
    ) -> "FinancingOffer":
        """
        Acknowledges that platform has received and delivered the financing_offer to
        the intended merchant recipient.
        """
        ...

    @overload
    async def mark_delivered_async(
        self, **params: Unpack["FinancingOfferMarkDeliveredParams"]
    ) -> "FinancingOffer":
        """
        Acknowledges that platform has received and delivered the financing_offer to
        the intended merchant recipient.
        """
        ...

    @class_method_variant("_cls_mark_delivered_async")
    async def mark_delivered_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["FinancingOfferMarkDeliveredParams"]
    ) -> "FinancingOffer":
        """
        Acknowledges that platform has received and delivered the financing_offer to
        the intended merchant recipient.
        """
        return cast(
            "FinancingOffer",
            await self._request_async(
                "post",
                "/v1/capital/financing_offers/{financing_offer}/mark_delivered".format(
                    financing_offer=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["FinancingOfferRetrieveParams"]
    ) -> "FinancingOffer":
        """
        Get the details of the financing offer
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["FinancingOfferRetrieveParams"]
    ) -> "FinancingOffer":
        """
        Get the details of the financing offer
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    class TestHelpers(APIResourceTestHelpers["FinancingOffer"]):
        _resource_cls: Type["FinancingOffer"]

        @classmethod
        def create(
            cls, **params: Unpack["FinancingOfferCreateParams"]
        ) -> "FinancingOffer":
            """
            Creates a test financing offer for a connected account.
            """
            return cast(
                "FinancingOffer",
                cls._static_request(
                    "post",
                    "/v1/test_helpers/capital/financing_offers",
                    params=params,
                ),
            )

        @classmethod
        async def create_async(
            cls, **params: Unpack["FinancingOfferCreateParams"]
        ) -> "FinancingOffer":
            """
            Creates a test financing offer for a connected account.
            """
            return cast(
                "FinancingOffer",
                await cls._static_request_async(
                    "post",
                    "/v1/test_helpers/capital/financing_offers",
                    params=params,
                ),
            )

        @classmethod
        def _cls_refill(
            cls,
            financing_offer: str,
            **params: Unpack["FinancingOfferRefillParams"],
        ) -> "FinancingOffer":
            """
            Refills a test financing offer for a connected account.
            """
            return cast(
                "FinancingOffer",
                cls._static_request(
                    "post",
                    "/v1/test_helpers/capital/financing_offers/{financing_offer}/refill".format(
                        financing_offer=sanitize_id(financing_offer)
                    ),
                    params=params,
                ),
            )

        @overload
        @staticmethod
        def refill(
            financing_offer: str,
            **params: Unpack["FinancingOfferRefillParams"],
        ) -> "FinancingOffer":
            """
            Refills a test financing offer for a connected account.
            """
            ...

        @overload
        def refill(
            self, **params: Unpack["FinancingOfferRefillParams"]
        ) -> "FinancingOffer":
            """
            Refills a test financing offer for a connected account.
            """
            ...

        @class_method_variant("_cls_refill")
        def refill(  # pyright: ignore[reportGeneralTypeIssues]
            self, **params: Unpack["FinancingOfferRefillParams"]
        ) -> "FinancingOffer":
            """
            Refills a test financing offer for a connected account.
            """
            return cast(
                "FinancingOffer",
                self.resource._request(
                    "post",
                    "/v1/test_helpers/capital/financing_offers/{financing_offer}/refill".format(
                        financing_offer=sanitize_id(
                            self.resource._data.get("id")
                        )
                    ),
                    params=params,
                ),
            )

        @classmethod
        async def _cls_refill_async(
            cls,
            financing_offer: str,
            **params: Unpack["FinancingOfferRefillParams"],
        ) -> "FinancingOffer":
            """
            Refills a test financing offer for a connected account.
            """
            return cast(
                "FinancingOffer",
                await cls._static_request_async(
                    "post",
                    "/v1/test_helpers/capital/financing_offers/{financing_offer}/refill".format(
                        financing_offer=sanitize_id(financing_offer)
                    ),
                    params=params,
                ),
            )

        @overload
        @staticmethod
        async def refill_async(
            financing_offer: str,
            **params: Unpack["FinancingOfferRefillParams"],
        ) -> "FinancingOffer":
            """
            Refills a test financing offer for a connected account.
            """
            ...

        @overload
        async def refill_async(
            self, **params: Unpack["FinancingOfferRefillParams"]
        ) -> "FinancingOffer":
            """
            Refills a test financing offer for a connected account.
            """
            ...

        @class_method_variant("_cls_refill_async")
        async def refill_async(  # pyright: ignore[reportGeneralTypeIssues]
            self, **params: Unpack["FinancingOfferRefillParams"]
        ) -> "FinancingOffer":
            """
            Refills a test financing offer for a connected account.
            """
            return cast(
                "FinancingOffer",
                await self.resource._request_async(
                    "post",
                    "/v1/test_helpers/capital/financing_offers/{financing_offer}/refill".format(
                        financing_offer=sanitize_id(
                            self.resource._data.get("id")
                        )
                    ),
                    params=params,
                ),
            )

    @property
    def test_helpers(self):
        return self.TestHelpers(self)

    _inner_class_types = {
        "accepted_terms": AcceptedTerms,
        "offered_terms": OfferedTerms,
    }


FinancingOffer.TestHelpers._resource_cls = FinancingOffer



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe.v2._amount import AmountParam
from typing_extensions import NotRequired, TypedDict

_CurrencyConversionCreateParamsBase = TypedDict(
    "CurrencyConversionCreateParams",
    {"from": "CurrencyConversionCreateParamsFrom"},
)


class CurrencyConversionCreateParams(_CurrencyConversionCreateParamsBase):
    financial_account: NotRequired[str]
    """
    The FinancialAccount id to create the CurrencyConversion on.
    """
    fx_quote: NotRequired[str]
    """
    The FX quote to use for the conversion.
    """
    to: "CurrencyConversionCreateParamsTo"
    """
    To amount object indicating the to currency or optional amount.
    """


class CurrencyConversionCreateParamsFrom(TypedDict):
    amount: NotRequired[AmountParam]
    """
    Amount object.
    """
    currency: NotRequired[str]
    """
    A lowercase alpha3 currency code like "usd".
    """


class CurrencyConversionCreateParamsTo(TypedDict):
    amount: NotRequired[AmountParam]
    """
    Amount object.
    """
    currency: NotRequired[str]
    """
    A lowercase alpha3 currency code like "usd".
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe.params.reserve._hold_list_params import HoldListParams
    from stripe.params.reserve._hold_retrieve_params import HoldRetrieveParams
    from stripe.reserve._hold import Hold


class HoldService(StripeService):
    def list(
        self,
        params: Optional["HoldListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Hold]":
        """
        Returns a list of ReserveHolds previously created. The ReserveHolds are returned in sorted order, with the most recent ReserveHolds appearing first.
        """
        return cast(
            "ListObject[Hold]",
            self._request(
                "get",
                "/v1/reserve/holds",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["HoldListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Hold]":
        """
        Returns a list of ReserveHolds previously created. The ReserveHolds are returned in sorted order, with the most recent ReserveHolds appearing first.
        """
        return cast(
            "ListObject[Hold]",
            await self._request_async(
                "get",
                "/v1/reserve/holds",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["HoldRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Hold":
        """
        Retrieve a ReserveHold.
        """
        return cast(
            "Hold",
            self._request(
                "get",
                "/v1/reserve/holds/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["HoldRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Hold":
        """
        Retrieve a ReserveHold.
        """
        return cast(
            "Hold",
            await self._request_async(
                "get",
                "/v1/reserve/holds/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe._setup_intent import SetupIntent
    from stripe.params._setup_intent_cancel_params import (
        SetupIntentCancelParams,
    )
    from stripe.params._setup_intent_confirm_params import (
        SetupIntentConfirmParams,
    )
    from stripe.params._setup_intent_create_params import (
        SetupIntentCreateParams,
    )
    from stripe.params._setup_intent_list_params import SetupIntentListParams
    from stripe.params._setup_intent_retrieve_params import (
        SetupIntentRetrieveParams,
    )
    from stripe.params._setup_intent_update_params import (
        SetupIntentUpdateParams,
    )
    from stripe.params._setup_intent_verify_microdeposits_params import (
        SetupIntentVerifyMicrodepositsParams,
    )


class SetupIntentService(StripeService):
    def list(
        self,
        params: Optional["SetupIntentListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[SetupIntent]":
        """
        Returns a list of SetupIntents.
        """
        return cast(
            "ListObject[SetupIntent]",
            self._request(
                "get",
                "/v1/setup_intents",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["SetupIntentListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[SetupIntent]":
        """
        Returns a list of SetupIntents.
        """
        return cast(
            "ListObject[SetupIntent]",
            await self._request_async(
                "get",
                "/v1/setup_intents",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: Optional["SetupIntentCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "SetupIntent":
        """
        Creates a SetupIntent object.

        After you create the SetupIntent, attach a payment method and [confirm](https://docs.stripe.com/docs/api/setup_intents/confirm)
        it to collect any required permissions to charge the payment method later.
        """
        return cast(
            "SetupIntent",
            self._request(
                "post",
                "/v1/setup_intents",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: Optional["SetupIntentCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "SetupIntent":
        """
        Creates a SetupIntent object.

        After you create the SetupIntent, attach a payment method and [confirm](https://docs.stripe.com/docs/api/setup_intents/confirm)
        it to collect any required permissions to charge the payment method later.
        """
        return cast(
            "SetupIntent",
            await self._request_async(
                "post",
                "/v1/setup_intents",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        intent: str,
        params: Optional["SetupIntentRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "SetupIntent":
        """
        Retrieves the details of a SetupIntent that has previously been created.

        Client-side retrieval using a publishable key is allowed when the client_secret is provided in the query string.

        When retrieved with a publishable key, only a subset of properties will be returned. Please refer to the [SetupIntent](https://docs.stripe.com/api#setup_intent_object) object reference for more details.
        """
        return cast(
            "SetupIntent",
            self._request(
                "get",
                "/v1/setup_intents/{intent}".format(
                    intent=sanitize_id(intent)
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        intent: str,
        params: Optional["SetupIntentRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "SetupIntent":
        """
        Retrieves the details of a SetupIntent that has previously been created.

        Client-side retrieval using a publishable key is allowed when the client_secret is provided in the query string.

        When retrieved with a publishable key, only a subset of properties will be returned. Please refer to the [SetupIntent](https://docs.stripe.com/api#setup_intent_object) object reference for more details.
        """
        return cast(
            "SetupIntent",
            await self._request_async(
                "get",
                "/v1/setup_intents/{intent}".format(
                    intent=sanitize_id(intent)
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        intent: str,
        params: Optional["SetupIntentUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "SetupIntent":
        """
        Updates a SetupIntent object.
        """
        return cast(
            "SetupIntent",
            self._request(
                "post",
                "/v1/setup_intents/{intent}".format(
                    intent=sanitize_id(intent)
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        intent: str,
        params: Optional["SetupIntentUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "SetupIntent":
        """
        Updates a SetupIntent object.
        """
        return cast(
            "SetupIntent",
            await self._request_async(
                "post",
                "/v1/setup_intents/{intent}".format(
                    intent=sanitize_id(intent)
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def cancel(
        self,
        intent: str,
        params: Optional["SetupIntentCancelParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "SetupIntent":
        """
        You can cancel a SetupIntent object when it's in one of these statuses: requires_payment_method, requires_confirmation, or requires_action.

        After you cancel it, setup is abandoned and any operations on the SetupIntent fail with an error. You can't cancel the SetupIntent for a Checkout Session. [Expire the Checkout Session](https://docs.stripe.com/docs/api/checkout/sessions/expire) instead.
        """
        return cast(
            "SetupIntent",
            self._request(
                "post",
                "/v1/setup_intents/{intent}/cancel".format(
                    intent=sanitize_id(intent),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def cancel_async(
        self,
        intent: str,
        params: Optional["SetupIntentCancelParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "SetupIntent":
        """
        You can cancel a SetupIntent object when it's in one of these statuses: requires_payment_method, requires_confirmation, or requires_action.

        After you cancel it, setup is abandoned and any operations on the SetupIntent fail with an error. You can't cancel the SetupIntent for a Checkout Session. [Expire the Checkout Session](https://docs.stripe.com/docs/api/checkout/sessions/expire) instead.
        """
        return cast(
            "SetupIntent",
            await self._request_async(
                "post",
                "/v1/setup_intents/{intent}/cancel".format(
                    intent=sanitize_id(intent),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def confirm(
        self,
        intent: str,
        params: Optional["SetupIntentConfirmParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "SetupIntent":
        """
        Confirm that your customer intends to set up the current or
        provided payment method. For example, you would confirm a SetupIntent
        when a customer hits the “Save” button on a payment method management
        page on your website.

        If the selected payment method does not require any additional
        steps from the customer, the SetupIntent will transition to the
        succeeded status.

        Otherwise, it will transition to the requires_action status and
        suggest additional actions via next_action. If setup fails,
        the SetupIntent will transition to the
        requires_payment_method status or the canceled status if the
        confirmation limit is reached.
        """
        return cast(
            "SetupIntent",
            self._request(
                "post",
                "/v1/setup_intents/{intent}/confirm".format(
                    intent=sanitize_id(intent),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def confirm_async(
        self,
        intent: str,
        params: Optional["SetupIntentConfirmParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "SetupIntent":
        """
        Confirm that your customer intends to set up the current or
        provided payment method. For example, you would confirm a SetupIntent
        when a customer hits the “Save” button on a payment method management
        page on your website.

        If the selected payment method does not require any additional
        steps from the customer, the SetupIntent will transition to the
        succeeded status.

        Otherwise, it will transition to the requires_action status and
        suggest additional actions via next_action. If setup fails,
        the SetupIntent will transition to the
        requires_payment_method status or the canceled status if the
        confirmation limit is reached.
        """
        return cast(
            "SetupIntent",
            await self._request_async(
                "post",
                "/v1/setup_intents/{intent}/confirm".format(
                    intent=sanitize_id(intent),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def verify_microdeposits(
        self,
        intent: str,
        params: Optional["SetupIntentVerifyMicrodepositsParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "SetupIntent":
        """
        Verifies microdeposits on a SetupIntent object.
        """
        return cast(
            "SetupIntent",
            self._request(
                "post",
                "/v1/setup_intents/{intent}/verify_microdeposits".format(
                    intent=sanitize_id(intent),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def verify_microdeposits_async(
        self,
        intent: str,
        params: Optional["SetupIntentVerifyMicrodepositsParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "SetupIntent":
        """
        Verifies microdeposits on a SetupIntent object.
        """
        return cast(
            "SetupIntent",
            await self._request_async(
                "post",
                "/v1/setup_intents/{intent}/verify_microdeposits".format(
                    intent=sanitize_id(intent),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from decimal import Decimal
from stripe._stripe_object import StripeObject, UntypedStripeObject
from typing import ClassVar, List, Optional
from typing_extensions import Literal


class IntentAction(StripeObject):
    """
    A Billing Intent Action represents a specific operation within a Billing Intent, such as subscribing to a Pricing Plan,
    modifying a subscription's quantity, or deactivating service. Each action has a specific type and associated details that
    define what change will be made when the Intent is committed.
    """

    OBJECT_NAME: ClassVar[Literal["v2.billing.intent_action"]] = (
        "v2.billing.intent_action"
    )

    class Apply(StripeObject):
        class Discount(StripeObject):
            coupon: Optional[str]
            """
            The ID of the Coupon applied.
            """
            discount: Optional[str]
            """
            The ID of the created Discount.
            """
            promotion_code: Optional[str]
            """
            The ID of the PromotionCode applied.
            """
            type: Literal["coupon", "promotion_code"]
            """
            Type of the discount.
            """

        class EffectiveAt(StripeObject):
            timestamp: Optional[str]
            """
            The timestamp at which the apply action takes effect. Only present if type is timestamp. Only allowed for discount actions.
            """
            type: Literal[
                "current_billing_period_end",
                "current_billing_period_start",
                "next_billing_period_start",
                "on_reserve",
                "timestamp",
            ]
            """
            When the apply action takes effect.
            """

        class InvoiceDiscountRule(StripeObject):
            class PercentOff(StripeObject):
                class MaximumApplications(StripeObject):
                    type: Literal["indefinite"]
                    """
                    The type of maximum applications configuration.
                    """

                maximum_applications: MaximumApplications
                """
                The maximum number of times this discount can be applied for this Billing Cadence.
                """
                percent_off: Decimal
                """
                Percent that is taken off the amount. For example, a percent_off of 50.0 reduces a 100 USD amount to 50 USD.
                """
                _inner_class_types = {
                    "maximum_applications": MaximumApplications,
                }
                _field_encodings = {"percent_off": "decimal_string"}

            applies_to: Literal["cadence"]
            """
            The entity that the discount rule applies to, for example, the Billing Cadence.
            """
            invoice_discount_rule: Optional[str]
            """
            The ID of the created discount rule. This is only present once the Billing Intent is committed and the discount rule is created.
            """
            percent_off: Optional[PercentOff]
            """
            Configuration for percentage off discount.
            """
            type: Literal["percent_off"]
            """
            Type of the discount rule.
            """
            _inner_class_types = {"percent_off": PercentOff}

        class SpendModifierRule(StripeObject):
            class MaxBillingPeriodSpend(StripeObject):
                class Amount(StripeObject):
                    class CustomPricingUnit(StripeObject):
                        id: Optional[str]
                        """
                        The id of the custom pricing unit.
                        """
                        value: str
                        """
                        The value of the custom pricing unit.
                        """

                    custom_pricing_unit: Optional[CustomPricingUnit]
                    """
                    The custom pricing unit amount.
                    """
                    type: Literal["custom_pricing_unit"]
                    """
                    The type of the amount.
                    """
                    _inner_class_types = {
                        "custom_pricing_unit": CustomPricingUnit,
                    }

                class CustomPricingUnitOverageRate(StripeObject):
                    id: str
                    """
                    ID of the custom pricing unit overage rate.
                    """

                amount: Amount
                """
                The maximum amount allowed for the billing period.
                """
                custom_pricing_unit_overage_rate: CustomPricingUnitOverageRate
                """
                The configuration for the overage rate for the custom pricing unit.
                """
                _inner_class_types = {
                    "amount": Amount,
                    "custom_pricing_unit_overage_rate": CustomPricingUnitOverageRate,
                }

            applies_to: Literal["cadence"]
            """
            What the spend modifier applies to.
            """
            id: str
            """
            The ID of the spend modifier.
            """
            max_billing_period_spend: Optional[MaxBillingPeriodSpend]
            """
            Details for max billing period spend modifier. Only present if type is max_billing_period_spend.
            """
            type: Literal["max_billing_period_spend"]
            """
            Type of the spend modifier.
            """
            _inner_class_types = {
                "max_billing_period_spend": MaxBillingPeriodSpend,
            }

        discount: Optional[Discount]
        """
        Details for applying a discount.
        """
        effective_at: Optional[EffectiveAt]
        """
        When the apply action takes effect. If not specified, defaults to on_reserve.
        """
        invoice_discount_rule: Optional[InvoiceDiscountRule]
        """
        Details for applying a discount rule to future invoices.
        """
        spend_modifier_rule: Optional[SpendModifierRule]
        """
        Details for applying a spend modifier rule. Only present if type is spend_modifier_rule.
        """
        type: Literal[
            "discount", "invoice_discount_rule", "spend_modifier_rule"
        ]
        """
        Type of the apply action details.
        """
        _inner_class_types = {
            "discount": Discount,
            "effective_at": EffectiveAt,
            "invoice_discount_rule": InvoiceDiscountRule,
            "spend_modifier_rule": SpendModifierRule,
        }

    class Deactivate(StripeObject):
        class CancellationDetails(StripeObject):
            comment: Optional[str]
            """
            Additional comments about why the user canceled the subscription, if the subscription was canceled explicitly by the user.
            """
            feedback: Optional[
                Literal[
                    "customer_service",
                    "low_quality",
                    "missing_features",
                    "other",
                    "switched_service",
                    "too_complex",
                    "too_expensive",
                    "unused",
                ]
            ]
            """
            The customer submitted reason for why they canceled, if the subscription was canceled explicitly by the user.
            """

        class EffectiveAt(StripeObject):
            timestamp: Optional[str]
            """
            The timestamp at which the deactivate action takes effect. Only present if type is timestamp.
            """
            type: Literal[
                "current_billing_period_end",
                "current_billing_period_start",
                "on_reserve",
                "timestamp",
            ]
            """
            When the deactivate action takes effect.
            """

        class PricingPlanSubscriptionDetails(StripeObject):
            class Overrides(StripeObject):
                class PartialPeriodBehavior(StripeObject):
                    class LicenseFee(StripeObject):
                        credit_proration_behavior: Literal["none", "prorated"]
                        """
                        The proration behavior for the partial servicing period. Defines how we prorate the license fee when the user is deactivating. If not specified, defaults to none.
                        """

                    license_fee: Optional[LicenseFee]
                    """
                    Overrides the behavior for license fee components when the action takes effect during the service period.
                    """
                    type: Literal["license_fee", "recurring_credit_grant"]
                    """
                    The type of behavior to override.
                    """
                    _inner_class_types = {"license_fee": LicenseFee}

                partial_period_behaviors: List[PartialPeriodBehavior]
                """
                Configurations for behaviors when the action takes effect during the service period.
                """
                _inner_class_types = {
                    "partial_period_behaviors": PartialPeriodBehavior,
                }

            overrides: Optional[Overrides]
            """
            Configurations for overriding behaviors related to the subscription.
            """
            pricing_plan_subscription: str
            """
            ID of the Pricing Plan Subscription to deactivate.
            """
            _inner_class_types = {"overrides": Overrides}

        cancellation_details: Optional[CancellationDetails]
        """
        Details about why the cancellation was requested by the user.
        """
        collect_at: Literal["next_billing_date", "on_effective_at"]
        """
        Allows users to override the collect at behavior.
        """
        effective_at: EffectiveAt
        """
        When the deactivate action takes effect. If not specified, the default behavior is on_reserve.
        """
        pricing_plan_subscription_details: Optional[
            PricingPlanSubscriptionDetails
        ]
        """
        Details for deactivating a Pricing Plan Subscription.
        """
        type: Literal[
            "pricing_plan_subscription_details", "v1_subscription_details"
        ]
        """
        Type of the action details.
        """
        _inner_class_types = {
            "cancellation_details": CancellationDetails,
            "effective_at": EffectiveAt,
            "pricing_plan_subscription_details": PricingPlanSubscriptionDetails,
        }

    class Modify(StripeObject):
        class EffectiveAt(StripeObject):
            timestamp: Optional[str]
            """
            The timestamp at which the modify action takes effect. Only present if type is timestamp.
            """
            type: Literal[
                "current_billing_period_start", "on_reserve", "timestamp"
            ]
            """
            When the modify action takes effect.
            """

        class PricingPlanSubscriptionDetails(StripeObject):
            class ComponentConfiguration(StripeObject):
                lookup_key: Optional[str]
                """
                Lookup key for the pricing plan component.
                """
                pricing_plan_component: Optional[str]
                """
                ID of the pricing plan component.
                """
                quantity: Optional[int]
                """
                Quantity of the component to be used.
                """

            class Overrides(StripeObject):
                class PartialPeriodBehavior(StripeObject):
                    class LicenseFee(StripeObject):
                        credit_proration_behavior: Literal["none", "prorated"]
                        """
                        The proration behavior for the partial servicing period. Defines how we prorate the license fee when the user modifies the subscription. If not specified, defaults to prorated.
                        """
                        debit_proration_behavior: Literal["none", "prorated"]
                        """
                        The proration behavior for the partial servicing period. Defines how we prorate the license fee when the user modifies the subscription. If not specified, defaults to prorated.
                        """

                    class RecurringCreditGrant(StripeObject):
                        create_behavior: Literal["full_credits", "none"]
                        """
                        Controls credit grant creation behavior during partial periods. If not specified, defaults to full_credits.
                        """

                    license_fee: Optional[LicenseFee]
                    """
                    Overrides the behavior for license fee components when the action takes effect during the service period.
                    """
                    recurring_credit_grant: Optional[RecurringCreditGrant]
                    """
                    Overrides the behavior for recurring credit grant components when the action takes effect during the service period.
                    """
                    type: Literal["license_fee", "recurring_credit_grant"]
                    """
                    The type of behavior to override.
                    """
                    _inner_class_types = {
                        "license_fee": LicenseFee,
                        "recurring_credit_grant": RecurringCreditGrant,
                    }

                partial_period_behaviors: List[PartialPeriodBehavior]
                """
                Configurations for behaviors when the action takes effect during the service period.
                """
                _inner_class_types = {
                    "partial_period_behaviors": PartialPeriodBehavior,
                }

            component_configurations: List[ComponentConfiguration]
            """
            New configurations for the components of the Pricing Plan.
            """
            new_pricing_plan: str
            """
            ID of the new Pricing Plan.
            """
            new_pricing_plan_version: str
            """
            Version of the Pricing Plan to use.
            """
            overrides: Optional[Overrides]
            """
            Configurations for overriding behaviors related to the subscription.
            """
            pricing_plan_subscription: str
            """
            ID of the Pricing Plan Subscription to modify.
            """
            _inner_class_types = {
                "component_configurations": ComponentConfiguration,
                "overrides": Overrides,
            }

        collect_at: Literal["next_billing_date", "on_effective_at"]
        """
        Allows users to override the collect at behavior.
        """
        effective_at: EffectiveAt
        """
        When the modify action takes effect. If not specified, the default behavior is on_reserve.
        """
        pricing_plan_subscription_details: Optional[
            PricingPlanSubscriptionDetails
        ]
        """
        Details for modifying a Pricing Plan Subscription.
        """
        type: Literal[
            "pricing_plan_subscription_details", "v1_subscription_details"
        ]
        """
        Type of the action details.
        """
        _inner_class_types = {
            "effective_at": EffectiveAt,
            "pricing_plan_subscription_details": PricingPlanSubscriptionDetails,
        }

    class Remove(StripeObject):
        class EffectiveAt(StripeObject):
            type: Literal["current_billing_period_end", "on_reserve"]
            """
            When the remove action takes effect.
            """

        effective_at: Optional[EffectiveAt]
        """
        When the remove action takes effect. If not specified, defaults to on_reserve.
        """
        invoice_discount_rule: Optional[str]
        """
        The ID of the discount rule to remove for future invoices.
        """
        spend_modifier_rule: Optional[str]
        """
        The ID of the spend modifier rule removed.
        """
        type: Literal["invoice_discount_rule", "spend_modifier_rule"]
        """
        Type of the remove action.
        """
        _inner_class_types = {"effective_at": EffectiveAt}

    class Subscribe(StripeObject):
        class EffectiveAt(StripeObject):
            timestamp: Optional[str]
            """
            The timestamp at which the subscribe action takes effect. Only present if type is timestamp.
            """
            type: Literal[
                "current_billing_period_start", "on_reserve", "timestamp"
            ]
            """
            When the subscribe action takes effect.
            """

        class PricingPlanSubscriptionDetails(StripeObject):
            class ComponentConfiguration(StripeObject):
                lookup_key: Optional[str]
                """
                Lookup key for the pricing plan component.
                """
                pricing_plan_component: Optional[str]
                """
                ID of the pricing plan component.
                """
                quantity: Optional[int]
                """
                Quantity of the component to be used.
                """

            class Overrides(StripeObject):
                class PartialPeriodBehavior(StripeObject):
                    class LicenseFee(StripeObject):
                        debit_proration_behavior: Literal["none", "prorated"]
                        """
                        The proration behavior for the partial servicing period. Defines how we prorate the license fee when the user is subscribing. If not specified, defaults to prorated.
                        """

                    class RecurringCreditGrant(StripeObject):
                        create_behavior: Literal["full_credits", "none"]
                        """
                        Controls credit grant creation behavior during partial periods. If not specified, defaults to full_credits.
                        """

                    license_fee: Optional[LicenseFee]
                    """
                    Overrides the behavior for license fee components when the action takes effect during the service period.
                    """
                    recurring_credit_grant: Optional[RecurringCreditGrant]
                    """
                    Overrides the behavior for recurring credit grant components when the action takes effect during the service period.
                    """
                    type: Literal["license_fee", "recurring_credit_grant"]
                    """
                    The type of behavior to override.
                    """
                    _inner_class_types = {
                        "license_fee": LicenseFee,
                        "recurring_credit_grant": RecurringCreditGrant,
                    }

                partial_period_behaviors: List[PartialPeriodBehavior]
                """
                Configurations for behaviors when the action takes effect during the service period.
                """
                _inner_class_types = {
                    "partial_period_behaviors": PartialPeriodBehavior,
                }

            component_configurations: List[ComponentConfiguration]
            """
            Configurations for the components of the Pricing Plan.
            """
            metadata: Optional[UntypedStripeObject[str]]
            """
            Set of [key-value pairs](https://docs.stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
            """
            overrides: Optional[Overrides]
            """
            Configurations for overriding behaviors related to the subscription.
            """
            pricing_plan: str
            """
            ID of the Pricing Plan to subscribe to.
            """
            pricing_plan_subscription: Optional[str]
            """
            ID of the created Pricing Plan Subscription. This is only present once the Billing Intent is committed and the Pricing Plan Subscription is created.
            """
            pricing_plan_version: str
            """
            Version of the Pricing Plan to use.
            """
            _inner_class_types = {
                "component_configurations": ComponentConfiguration,
                "overrides": Overrides,
            }

        class V1SubscriptionDetails(StripeObject):
            class Item(StripeObject):
                metadata: Optional[UntypedStripeObject[str]]
                """
                Set of key-value pairs that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
                """
                price: str
                """
                The ID of the price object.
                """
                quantity: Optional[int]
                """
                Quantity for this item. If not provided, defaults to 1.
                """

            description: Optional[str]
            """
            The subscription's description, meant to be displayable to the customer.
            Use this field to optionally store an explanation of the subscription for rendering in Stripe surfaces and certain local payment methods UIs.
            """
            items: List[Item]
            """
            A list of up to 20 subscription items, each with an attached price.
            """
            metadata: Optional[UntypedStripeObject[str]]
            """
            Set of key-value pairs that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
            """
            _inner_class_types = {"items": Item}

        collect_at: Literal["next_billing_date", "on_effective_at"]
        """
        Allows users to override the collect at behavior.
        """
        effective_at: EffectiveAt
        """
        When the subscribe action takes effect. If not specified, the default behavior is on_reserve.
        """
        pricing_plan_subscription_details: Optional[
            PricingPlanSubscriptionDetails
        ]
        """
        Details for subscribing to a Pricing Plan.
        """
        type: Literal[
            "pricing_plan_subscription_details", "v1_subscription_details"
        ]
        """
        Type of the action details.
        """
        v1_subscription_details: Optional[V1SubscriptionDetails]
        """
        Details for subscribing to a V1 subscription.
        """
        _inner_class_types = {
            "effective_at": EffectiveAt,
            "pricing_plan_subscription_details": PricingPlanSubscriptionDetails,
            "v1_subscription_details": V1SubscriptionDetails,
        }

    apply: Optional[Apply]
    """
    Details for an apply action.
    """
    created: str
    """
    Time at which the object was created.
    """
    deactivate: Optional[Deactivate]
    """
    Details for a deactivate action.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    modify: Optional[Modify]
    """
    Details for a modify action.
    """
    object: Literal["v2.billing.intent_action"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    remove: Optional[Remove]
    """
    Details for a remove action.
    """
    subscribe: Optional[Subscribe]
    """
    Details for a subscribe action.
    """
    type: Literal["apply", "deactivate", "modify", "remove", "subscribe"]
    """
    Type of the Billing Intent Action.
    """
    _inner_class_types = {
        "apply": Apply,
        "deactivate": Deactivate,
        "modify": Modify,
        "remove": Remove,
        "subscribe": Subscribe,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.treasury._credit_reversal import (
        CreditReversal as CreditReversal,
    )
    from stripe.treasury._credit_reversal_service import (
        CreditReversalService as CreditReversalService,
    )
    from stripe.treasury._debit_reversal import DebitReversal as DebitReversal
    from stripe.treasury._debit_reversal_service import (
        DebitReversalService as DebitReversalService,
    )
    from stripe.treasury._financial_account import (
        FinancialAccount as FinancialAccount,
    )
    from stripe.treasury._financial_account_features import (
        FinancialAccountFeatures as FinancialAccountFeatures,
    )
    from stripe.treasury._financial_account_features_service import (
        FinancialAccountFeaturesService as FinancialAccountFeaturesService,
    )
    from stripe.treasury._financial_account_service import (
        FinancialAccountService as FinancialAccountService,
    )
    from stripe.treasury._inbound_transfer import (
        InboundTransfer as InboundTransfer,
    )
    from stripe.treasury._inbound_transfer_service import (
        InboundTransferService as InboundTransferService,
    )
    from stripe.treasury._outbound_payment import (
        OutboundPayment as OutboundPayment,
    )
    from stripe.treasury._outbound_payment_service import (
        OutboundPaymentService as OutboundPaymentService,
    )
    from stripe.treasury._outbound_transfer import (
        OutboundTransfer as OutboundTransfer,
    )
    from stripe.treasury._outbound_transfer_service import (
        OutboundTransferService as OutboundTransferService,
    )
    from stripe.treasury._received_credit import (
        ReceivedCredit as ReceivedCredit,
    )
    from stripe.treasury._received_credit_service import (
        ReceivedCreditService as ReceivedCreditService,
    )
    from stripe.treasury._received_debit import ReceivedDebit as ReceivedDebit
    from stripe.treasury._received_debit_service import (
        ReceivedDebitService as ReceivedDebitService,
    )
    from stripe.treasury._transaction import Transaction as Transaction
    from stripe.treasury._transaction_entry import (
        TransactionEntry as TransactionEntry,
    )
    from stripe.treasury._transaction_entry_service import (
        TransactionEntryService as TransactionEntryService,
    )
    from stripe.treasury._transaction_service import (
        TransactionService as TransactionService,
    )

# name -> (import_target, is_submodule)
_import_map = {
    "CreditReversal": ("stripe.treasury._credit_reversal", False),
    "CreditReversalService": (
        "stripe.treasury._credit_reversal_service",
        False,
    ),
    "DebitReversal": ("stripe.treasury._debit_reversal", False),
    "DebitReversalService": ("stripe.treasury._debit_reversal_service", False),
    "FinancialAccount": ("stripe.treasury._financial_account", False),
    "FinancialAccountFeatures": (
        "stripe.treasury._financial_account_features",
        False,
    ),
    "FinancialAccountFeaturesService": (
        "stripe.treasury._financial_account_features_service",
        False,
    ),
    "FinancialAccountService": (
        "stripe.treasury._financial_account_service",
        False,
    ),
    "InboundTransfer": ("stripe.treasury._inbound_transfer", False),
    "InboundTransferService": (
        "stripe.treasury._inbound_transfer_service",
        False,
    ),
    "OutboundPayment": ("stripe.treasury._outbound_payment", False),
    "OutboundPaymentService": (
        "stripe.treasury._outbound_payment_service",
        False,
    ),
    "OutboundTransfer": ("stripe.treasury._outbound_transfer", False),
    "OutboundTransferService": (
        "stripe.treasury._outbound_transfer_service",
        False,
    ),
    "ReceivedCredit": ("stripe.treasury._received_credit", False),
    "ReceivedCreditService": (
        "stripe.treasury._received_credit_service",
        False,
    ),
    "ReceivedDebit": ("stripe.treasury._received_debit", False),
    "ReceivedDebitService": ("stripe.treasury._received_debit_service", False),
    "Transaction": ("stripe.treasury._transaction", False),
    "TransactionEntry": ("stripe.treasury._transaction_entry", False),
    "TransactionEntryService": (
        "stripe.treasury._transaction_entry_service",
        False,
    ),
    "TransactionService": ("stripe.treasury._transaction_service", False),
}
if not TYPE_CHECKING:

    def __getattr__(name):
        try:
            target, is_submodule = _import_map[name]
            module = import_module(target)
            if is_submodule:
                return module

            return getattr(
                module,
                name,
            )
        except KeyError:
            raise AttributeError()



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._payout import Payout
    from stripe._request_options import RequestOptions
    from stripe.params._payout_cancel_params import PayoutCancelParams
    from stripe.params._payout_create_params import PayoutCreateParams
    from stripe.params._payout_list_params import PayoutListParams
    from stripe.params._payout_retrieve_params import PayoutRetrieveParams
    from stripe.params._payout_reverse_params import PayoutReverseParams
    from stripe.params._payout_update_params import PayoutUpdateParams


class PayoutService(StripeService):
    def list(
        self,
        params: Optional["PayoutListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Payout]":
        """
        Returns a list of existing payouts sent to third-party bank accounts or payouts that Stripe sent to you. The payouts return in sorted order, with the most recently created payouts appearing first.
        """
        return cast(
            "ListObject[Payout]",
            self._request(
                "get",
                "/v1/payouts",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["PayoutListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Payout]":
        """
        Returns a list of existing payouts sent to third-party bank accounts or payouts that Stripe sent to you. The payouts return in sorted order, with the most recently created payouts appearing first.
        """
        return cast(
            "ListObject[Payout]",
            await self._request_async(
                "get",
                "/v1/payouts",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "PayoutCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Payout":
        """
        To send funds to your own bank account, create a new payout object. Your [Stripe balance](https://docs.stripe.com/api#balance) must cover the payout amount. If it doesn't, you receive an “Insufficient Funds” error.

        If your API key is in test mode, money won't actually be sent, though every other action occurs as if you're in live mode.

        If you create a manual payout on a Stripe account that uses multiple payment source types, you need to specify the source type balance that the payout draws from. The [balance object](https://docs.stripe.com/api/balances/object) details available and pending amounts by source type.
        """
        return cast(
            "Payout",
            self._request(
                "post",
                "/v1/payouts",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "PayoutCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Payout":
        """
        To send funds to your own bank account, create a new payout object. Your [Stripe balance](https://docs.stripe.com/api#balance) must cover the payout amount. If it doesn't, you receive an “Insufficient Funds” error.

        If your API key is in test mode, money won't actually be sent, though every other action occurs as if you're in live mode.

        If you create a manual payout on a Stripe account that uses multiple payment source types, you need to specify the source type balance that the payout draws from. The [balance object](https://docs.stripe.com/api/balances/object) details available and pending amounts by source type.
        """
        return cast(
            "Payout",
            await self._request_async(
                "post",
                "/v1/payouts",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        payout: str,
        params: Optional["PayoutRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Payout":
        """
        Retrieves the details of an existing payout. Supply the unique payout ID from either a payout creation request or the payout list. Stripe returns the corresponding payout information.
        """
        return cast(
            "Payout",
            self._request(
                "get",
                "/v1/payouts/{payout}".format(payout=sanitize_id(payout)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        payout: str,
        params: Optional["PayoutRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Payout":
        """
        Retrieves the details of an existing payout. Supply the unique payout ID from either a payout creation request or the payout list. Stripe returns the corresponding payout information.
        """
        return cast(
            "Payout",
            await self._request_async(
                "get",
                "/v1/payouts/{payout}".format(payout=sanitize_id(payout)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        payout: str,
        params: Optional["PayoutUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Payout":
        """
        Updates the specified payout by setting the values of the parameters you pass. We don't change parameters that you don't provide. This request only accepts the metadata as arguments.
        """
        return cast(
            "Payout",
            self._request(
                "post",
                "/v1/payouts/{payout}".format(payout=sanitize_id(payout)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        payout: str,
        params: Optional["PayoutUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Payout":
        """
        Updates the specified payout by setting the values of the parameters you pass. We don't change parameters that you don't provide. This request only accepts the metadata as arguments.
        """
        return cast(
            "Payout",
            await self._request_async(
                "post",
                "/v1/payouts/{payout}".format(payout=sanitize_id(payout)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def cancel(
        self,
        payout: str,
        params: Optional["PayoutCancelParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Payout":
        """
        You can cancel a previously created payout if its status is pending. Stripe refunds the funds to your available balance. You can't cancel automatic Stripe payouts.
        """
        return cast(
            "Payout",
            self._request(
                "post",
                "/v1/payouts/{payout}/cancel".format(
                    payout=sanitize_id(payout),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def cancel_async(
        self,
        payout: str,
        params: Optional["PayoutCancelParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Payout":
        """
        You can cancel a previously created payout if its status is pending. Stripe refunds the funds to your available balance. You can't cancel automatic Stripe payouts.
        """
        return cast(
            "Payout",
            await self._request_async(
                "post",
                "/v1/payouts/{payout}/cancel".format(
                    payout=sanitize_id(payout),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def reverse(
        self,
        payout: str,
        params: Optional["PayoutReverseParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Payout":
        """
        Reverses a payout by debiting the destination bank account. At this time, you can only reverse payouts for connected accounts to US and Canadian bank accounts. If the payout is manual and in the pending status, use /v1/payouts/:id/cancel instead.

        By requesting a reversal through /v1/payouts/:id/reverse, you confirm that the authorized signatory of the selected bank account authorizes the debit on the bank account and that no other authorization is required.
        """
        return cast(
            "Payout",
            self._request(
                "post",
                "/v1/payouts/{payout}/reverse".format(
                    payout=sanitize_id(payout),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def reverse_async(
        self,
        payout: str,
        params: Optional["PayoutReverseParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Payout":
        """
        Reverses a payout by debiting the destination bank account. At this time, you can only reverse payouts for connected accounts to US and Canadian bank accounts. If the payout is manual and in the pending status, use /v1/payouts/:id/cancel instead.

        By requesting a reversal through /v1/payouts/:id/reverse, you confirm that the authorized signatory of the selected bank account authorizes the debit on the bank account and that no other authorization is required.
        """
        return cast(
            "Payout",
            await self._request_async(
                "post",
                "/v1/payouts/{payout}/reverse".format(
                    payout=sanitize_id(payout),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from decimal import Decimal
from stripe._stripe_object import StripeObject
from typing import ClassVar, List, Optional
from typing_extensions import Literal


class AccountSignal(StripeObject):
    """
    An automatically evaluated signal on a v2 account.
    """

    OBJECT_NAME: ClassVar[Literal["v2.signals.account_signal"]] = (
        "v2.signals.account_signal"
    )

    class AccountDetails(StripeObject):
        account: Optional[str]
        """
        The v2 account ID of the account.
        """
        customer: Optional[str]
        """
        The v1 customer ID of the account, for users not yet migrated to v2/accounts.
        """

    class FraudulentMerchant(StripeObject):
        class Indicator(StripeObject):
            explanation: str
            """
            A brief explanation of how this indicator contributed to the fraudulent merchant probability.
            """
            impact: Literal[
                "decrease", "neutral", "slight_increase", "strong_increase"
            ]
            """
            The effect this indicator had on the overall risk level.
            """
            indicator: Literal[
                "bank_account",
                "business_information_and_account_activity",
                "disputes",
                "failures",
                "geolocation",
                "other",
                "other_related_accounts",
                "other_transaction_activity",
                "owner_email",
            ]
            """
            The name of the specific indicator used in the risk assessment.
            """

        indicators: List[Indicator]
        """
        Array of objects representing individual factors that contributed to the calculated probability. Absent when risk level is not_assessed or unknown,
        or when the user is not on a product tier that includes indicators.
        """
        probability: Optional[Decimal]
        """
        The probability of the merchant being fraudulent. Can be between 0.00 and 100.00. Absent when risk level is not_assessed or unknown,
        or when the user is not on a product tier that includes numeric scores.
        """
        risk_level: Literal[
            "elevated", "highest", "low", "normal", "not_assessed", "unknown"
        ]
        """
        Categorical assessment of the fraudulent merchant risk based on probability.
        """
        _inner_class_types = {"indicators": Indicator}
        _field_encodings = {"probability": "decimal_string"}

    class MerchantDelinquency(StripeObject):
        class Indicator(StripeObject):
            explanation: str
            """
            A brief explanation of how this indicator contributed to the delinquency probability.
            """
            impact: Literal[
                "decrease", "neutral", "slight_increase", "strong_increase"
            ]
            """
            The effect this indicator had on the overall risk level.
            """
            indicator: Literal[
                "account_balance",
                "aov",
                "charge_concentration",
                "disputes",
                "dispute_window",
                "exposure",
                "firmographic",
                "lifetime_metrics",
                "other",
                "payment_processing",
                "payment_volume",
                "payouts",
                "refunds",
                "related_accounts",
                "tenure",
                "transfers",
            ]
            """
            The name of the specific indicator used in the risk assessment.
            """

        indicators: List[Indicator]
        """
        Array of objects representing individual factors that contributed to the calculated probability of delinquency. Absent when risk level is not_assessed or unknown,
        or when the user is not on a product tier that includes indicators.
        """
        probability: Optional[Decimal]
        """
        The probability of delinquency. Can be between 0.00 and 100.00. Absent when risk level is not_assessed or unknown,
        or when the user is not on a product tier that includes numeric scores.
        """
        risk_level: Literal[
            "elevated", "highest", "low", "normal", "not_assessed", "unknown"
        ]
        """
        Categorical assessment of the delinquency risk based on probability.
        """
        _inner_class_types = {"indicators": Indicator}
        _field_encodings = {"probability": "decimal_string"}

    account_details: Optional[AccountDetails]
    """
    The account or customer this signal is associated with.
    """
    created: str
    """
    Timestamp at which the signal was created.
    """
    fraudulent_merchant: Optional[FraudulentMerchant]
    """
    Data for the fraudulent merchant signal. Present only when type is fraudulent_merchant.
    """
    id: str
    """
    Unique identifier for the account signal.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    merchant_delinquency: Optional[MerchantDelinquency]
    """
    Data for the merchant delinquency signal. Present only when type is merchant_delinquency.
    """
    object: Literal["v2.signals.account_signal"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    type: Literal["fraudulent_merchant", "merchant_delinquency"]
    """
    The type of signal.
    """
    _inner_class_types = {
        "account_details": AccountDetails,
        "fraudulent_merchant": FraudulentMerchant,
        "merchant_delinquency": MerchantDelinquency,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._createable_api_resource import CreateableAPIResource
from stripe._deletable_api_resource import DeletableAPIResource
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._stripe_object import StripeObject
from stripe._updateable_api_resource import UpdateableAPIResource
from stripe._util import class_method_variant, sanitize_id
from typing import ClassVar, List, Optional, cast, overload
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params._payment_location_create_params import (
        PaymentLocationCreateParams,
    )
    from stripe.params._payment_location_delete_params import (
        PaymentLocationDeleteParams,
    )
    from stripe.params._payment_location_list_params import (
        PaymentLocationListParams,
    )
    from stripe.params._payment_location_modify_params import (
        PaymentLocationModifyParams,
    )
    from stripe.params._payment_location_retrieve_params import (
        PaymentLocationRetrieveParams,
    )


class PaymentLocation(
    CreateableAPIResource["PaymentLocation"],
    DeletableAPIResource["PaymentLocation"],
    ListableAPIResource["PaymentLocation"],
    UpdateableAPIResource["PaymentLocation"],
):
    """
    A Payment Location represents a physical location where payments take place.
    """

    OBJECT_NAME: ClassVar[Literal["payment_location"]] = "payment_location"

    class Address(StripeObject):
        city: Optional[str]
        """
        City, district, suburb, town, or village.
        """
        country: Optional[str]
        """
        Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
        """
        line1: Optional[str]
        """
        Address line 1, such as the street, PO Box, or company name.
        """
        line2: Optional[str]
        """
        Address line 2, such as the apartment, suite, unit, or building.
        """
        postal_code: Optional[str]
        """
        ZIP or postal code.
        """
        state: Optional[str]
        """
        State, county, province, or region ([ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2)).
        """

    class BusinessRegistration(StripeObject):
        siret: Optional[str]
        """
        14-digit SIRET (Système d'identification du répertoire des établissements) number for the location.
        """

    class CapabilitySettings(StripeObject):
        class FrMealVouchersConecsPayments(StripeObject):
            class SupportedIssuers(StripeObject):
                card: Optional[
                    List[Literal["bimpli", "edenred", "pluxee", "up"]]
                ]
                """
                Supported meal voucher issuers for card payments.
                """
                card_present: Optional[
                    List[Literal["bimpli", "edenred", "pluxee", "up"]]
                ]
                """
                Supported meal voucher issuers for card present payments.
                """

            supported_issuers: Optional[SupportedIssuers]
            """
            Supported meal voucher issuers.
            """
            _inner_class_types = {"supported_issuers": SupportedIssuers}

        fr_meal_vouchers_conecs_payments: Optional[
            FrMealVouchersConecsPayments
        ]
        """
        Settings for Conecs French meal voucher capability.
        """
        _inner_class_types = {
            "fr_meal_vouchers_conecs_payments": FrMealVouchersConecsPayments,
        }

    address: Address
    business_registration: Optional[BusinessRegistration]
    """
    Identification numbers associated with the location.
    """
    capability_settings: Optional[CapabilitySettings]
    """
    The capability settings for the location. Only applicable for locations with requested payment location capabilities.
    """
    deleted: Optional[Literal[True]]
    """
    Always true for a deleted object
    """
    display_name: str
    """
    The display name of the location.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    object: Literal["payment_location"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """

    @classmethod
    def create(
        cls, **params: Unpack["PaymentLocationCreateParams"]
    ) -> "PaymentLocation":
        """
        Create a Payment Location.
        """
        return cast(
            "PaymentLocation",
            cls._static_request(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    async def create_async(
        cls, **params: Unpack["PaymentLocationCreateParams"]
    ) -> "PaymentLocation":
        """
        Create a Payment Location.
        """
        return cast(
            "PaymentLocation",
            await cls._static_request_async(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    def _cls_delete(
        cls, sid: str, **params: Unpack["PaymentLocationDeleteParams"]
    ) -> "PaymentLocation":
        """
        Delete a Payment Location.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(sid))
        return cast(
            "PaymentLocation",
            cls._static_request(
                "delete",
                url,
                params=params,
            ),
        )

    @overload
    @staticmethod
    def delete(
        sid: str, **params: Unpack["PaymentLocationDeleteParams"]
    ) -> "PaymentLocation":
        """
        Delete a Payment Location.
        """
        ...

    @overload
    def delete(
        self, **params: Unpack["PaymentLocationDeleteParams"]
    ) -> "PaymentLocation":
        """
        Delete a Payment Location.
        """
        ...

    @class_method_variant("_cls_delete")
    def delete(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["PaymentLocationDeleteParams"]
    ) -> "PaymentLocation":
        """
        Delete a Payment Location.
        """
        return self._request_and_refresh(
            "delete",
            self.instance_url(),
            params=params,
        )

    @classmethod
    async def _cls_delete_async(
        cls, sid: str, **params: Unpack["PaymentLocationDeleteParams"]
    ) -> "PaymentLocation":
        """
        Delete a Payment Location.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(sid))
        return cast(
            "PaymentLocation",
            await cls._static_request_async(
                "delete",
                url,
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def delete_async(
        sid: str, **params: Unpack["PaymentLocationDeleteParams"]
    ) -> "PaymentLocation":
        """
        Delete a Payment Location.
        """
        ...

    @overload
    async def delete_async(
        self, **params: Unpack["PaymentLocationDeleteParams"]
    ) -> "PaymentLocation":
        """
        Delete a Payment Location.
        """
        ...

    @class_method_variant("_cls_delete_async")
    async def delete_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["PaymentLocationDeleteParams"]
    ) -> "PaymentLocation":
        """
        Delete a Payment Location.
        """
        return await self._request_and_refresh_async(
            "delete",
            self.instance_url(),
            params=params,
        )

    @classmethod
    def list(
        cls, **params: Unpack["PaymentLocationListParams"]
    ) -> ListObject["PaymentLocation"]:
        """
        List all Payment Locations.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["PaymentLocationListParams"]
    ) -> ListObject["PaymentLocation"]:
        """
        List all Payment Locations.
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def modify(
        cls, id: str, **params: Unpack["PaymentLocationModifyParams"]
    ) -> "PaymentLocation":
        """
        Update a Payment Location.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "PaymentLocation",
            cls._static_request(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    async def modify_async(
        cls, id: str, **params: Unpack["PaymentLocationModifyParams"]
    ) -> "PaymentLocation":
        """
        Update a Payment Location.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "PaymentLocation",
            await cls._static_request_async(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["PaymentLocationRetrieveParams"]
    ) -> "PaymentLocation":
        """
        Retrieve a Payment Location.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["PaymentLocationRetrieveParams"]
    ) -> "PaymentLocation":
        """
        Retrieve a Payment Location.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    _inner_class_types = {
        "address": Address,
        "business_registration": BusinessRegistration,
        "capability_settings": CapabilitySettings,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe.apps._secret import Secret
    from stripe.params.apps._secret_create_params import SecretCreateParams
    from stripe.params.apps._secret_delete_where_params import (
        SecretDeleteWhereParams,
    )
    from stripe.params.apps._secret_find_params import SecretFindParams
    from stripe.params.apps._secret_list_params import SecretListParams


class SecretService(StripeService):
    def list(
        self,
        params: "SecretListParams",
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Secret]":
        """
        List all secrets stored on the given scope.
        """
        return cast(
            "ListObject[Secret]",
            self._request(
                "get",
                "/v1/apps/secrets",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: "SecretListParams",
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Secret]":
        """
        List all secrets stored on the given scope.
        """
        return cast(
            "ListObject[Secret]",
            await self._request_async(
                "get",
                "/v1/apps/secrets",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "SecretCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Secret":
        """
        Create or replace a secret in the secret store.
        """
        return cast(
            "Secret",
            self._request(
                "post",
                "/v1/apps/secrets",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "SecretCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Secret":
        """
        Create or replace a secret in the secret store.
        """
        return cast(
            "Secret",
            await self._request_async(
                "post",
                "/v1/apps/secrets",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def find(
        self,
        params: "SecretFindParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Secret":
        """
        Finds a secret in the secret store by name and scope.
        """
        return cast(
            "Secret",
            self._request(
                "get",
                "/v1/apps/secrets/find",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def find_async(
        self,
        params: "SecretFindParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Secret":
        """
        Finds a secret in the secret store by name and scope.
        """
        return cast(
            "Secret",
            await self._request_async(
                "get",
                "/v1/apps/secrets/find",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def delete_where(
        self,
        params: "SecretDeleteWhereParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Secret":
        """
        Deletes a secret from the secret store by name and scope.
        """
        return cast(
            "Secret",
            self._request(
                "post",
                "/v1/apps/secrets/delete",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def delete_where_async(
        self,
        params: "SecretDeleteWhereParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Secret":
        """
        Deletes a secret from the secret store by name and scope.
        """
        return cast(
            "Secret",
            await self._request_async(
                "post",
                "/v1/apps/secrets/delete",
                base_address="api",
                params=params,
                options=options,
            ),
        )
